#!/usr/bin/python
#
# Copyright (c) 2024, Florian Apolloner (@apollo13)
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: consul_token
short_description: Manipulate Consul tokens
version_added: 8.3.0
description:
  - Allows the addition, modification and deletion of tokens in a Consul cluster using the agent. For more details on using
    and configuring ACLs, see U(https://www.consul.io/docs/guides/acl.html).
author:
  - Florian Apolloner (@apollo13)
extends_documentation_fragment:
  - community.general._consul
  - community.general._consul.token
  - community.general._consul.actiongroup_consul
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: partial
    details:
      - In check mode the diff misses operational attributes.
  action_group:
    version_added: 8.3.0
options:
  state:
    description:
      - Whether the token should be present or absent.
    choices: ['present', 'absent']
    default: present
    type: str
  accessor_id:
    description:
      - Specifies a UUID to use as the token's Accessor ID. If not specified a UUID is generated for this field.
    type: str
  secret_id:
    description:
      - Specifies a UUID to use as the token's Secret ID. If not specified a UUID is generated for this field.
    type: str
  description:
    description:
      - Free form human readable description of the token.
    type: str
  policies:
    type: list
    elements: dict
    description:
      - List of policies to attach to the token. Each policy is a dict.
      - If the parameter is left blank, any policies currently assigned are not changed.
      - Any empty array (V([])) clears any policies previously set.
    suboptions:
      name:
        description:
          - The name of the policy to attach to this token; see M(community.general.consul_policy) for more info.
          - Either this or O(policies[].id) must be specified.
        type: str
      id:
        description:
          - The ID of the policy to attach to this token; see M(community.general.consul_policy) for more info.
          - Either this or O(policies[].name) must be specified.
        type: str
  roles:
    type: list
    elements: dict
    description:
      - List of roles to attach to the token. Each role is a dict.
      - If the parameter is left blank, any roles currently assigned are not changed.
      - Any empty array (V([])) clears any roles previously set.
    suboptions:
      name:
        description:
          - The name of the role to attach to this token; see M(community.general.consul_role) for more info.
          - Either this or O(roles[].id) must be specified.
        type: str
      id:
        description:
          - The ID of the role to attach to this token; see M(community.general.consul_role) for more info.
          - Either this or O(roles[].name) must be specified.
        type: str
  templated_policies:
    description:
      - The list of templated policies that should be applied to the role.
    type: list
    elements: dict
    suboptions:
      template_name:
        description:
          - The templated policy name.
        type: str
        required: true
      template_variables:
        description:
          - The templated policy variables.
          - Not all templated policies require variables.
        type: dict
  service_identities:
    type: list
    elements: dict
    description:
      - List of service identities to attach to the token.
      - If not specified, any service identities currently assigned are not changed.
      - If the parameter is an empty array (V([])), any node identities assigned are unassigned.
    suboptions:
      service_name:
        description:
          - The name of the service.
          - Must not be longer than 256 characters, must start and end with a lowercase alphanumeric character.
          - May only contain lowercase alphanumeric characters as well as V(-) and V(_).
        type: str
        required: true
      datacenters:
        description:
          - The datacenters where the token is effective.
          - If an empty array (V([])) is specified, the token is valid in all datacenters.
          - Including those which do not yet exist but may in the future.
        type: list
        elements: str
  node_identities:
    type: list
    elements: dict
    description:
      - List of node identities to attach to the token.
      - If not specified, any node identities currently assigned are not changed.
      - If the parameter is an empty array (V([])), any node identities assigned are unassigned.
    suboptions:
      node_name:
        description:
          - The name of the node.
          - Must not be longer than 256 characters, must start and end with a lowercase alphanumeric character.
          - May only contain lowercase alphanumeric characters as well as V(-) and V(_).
        type: str
        required: true
      datacenter:
        description:
          - The nodes datacenter.
          - This results in effective token only being valid in this datacenter.
        type: str
        required: true
  local:
    description:
      - If true, indicates that the token should not be replicated globally and instead be local to the current datacenter.
    type: bool
  expiration_ttl:
    description:
      - This is a convenience field and if set it initializes the C(expiration_time). Can be specified in the form of V(60s)
        or V(5m) (that is, 60 seconds or 5 minutes, respectively). Ingored when the token is updated!
    type: str
"""

EXAMPLES = r"""
- name: Create / Update a token by accessor_id
  community.general.consul_token:
    state: present
    accessor_id: 07a7de84-c9c7-448a-99cc-beaf682efd21
    token: 8adddd91-0bd6-d41d-ae1a-3b49cfa9a0e8
    roles:
      - name: role1
      - name: role2
    service_identities:
      - service_name: service1
        datacenters: [dc1, dc2]
    node_identities:
      - node_name: node1
        datacenter: dc1
    expiration_ttl: 50m

- name: Delete a token
  community.general.consul_token:
    state: absent
    accessor_id: 07a7de84-c9c7-448a-99cc-beaf682efd21
    token: 8adddd91-0bd6-d41d-ae1a-3b49cfa9a0e8
"""

RETURN = r"""
token:
  description: The token as returned by the Consul HTTP API.
  returned: always
  type: dict
  sample:
    AccessorID: 07a7de84-c9c7-448a-99cc-beaf682efd21
    CreateIndex: 632
    CreateTime: "2024-01-14T21:53:01.402749174+01:00"
    Description: Testing
    Hash: rj5PeDHddHslkpW7Ij4OD6N4bbSXiecXFmiw2SYXg2A=
    Local: false
    ModifyIndex: 633
    SecretID: bd380fba-da17-7cee-8576-8d6427c6c930
    ServiceIdentities: ["ServiceName": "test"]
operation:
  description: The operation performed.
  returned: changed
  type: str
  sample: update
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._consul import (
    AUTH_ARGUMENTS_SPEC,
    _ConsulModule,
)


def normalize_link_obj(api_obj, module_obj, key):
    api_objs = api_obj.get(key)
    module_objs = module_obj.get(key)
    if api_objs is None or module_objs is None:
        return
    name_to_id = {i["Name"]: i["ID"] for i in api_objs}
    id_to_name = {i["ID"]: i["Name"] for i in api_objs}

    for obj in module_objs:
        identifier = obj.get("ID")
        name = obj.get("Name")
        if identifier and not name and identifier in id_to_name:
            obj["Name"] = id_to_name[identifier]
        if not identifier and name and name in name_to_id:
            obj["ID"] = name_to_id[name]


class ConsulTokenModule(_ConsulModule):
    api_endpoint = "acl/token"
    result_key = "token"
    unique_identifiers = ["accessor_id"]

    create_only_fields = {"expiration_ttl"}

    def read_object(self):
        # if `accessor_id` is not supplied we can only create objects and are not idempotent
        if not self.id_from_obj(self.params):
            return None
        return super().read_object()

    def needs_update(self, api_obj, module_obj):
        # SecretID is usually not supplied
        if "SecretID" not in module_obj and "SecretID" in api_obj:
            del api_obj["SecretID"]
        normalize_link_obj(api_obj, module_obj, "Roles")
        normalize_link_obj(api_obj, module_obj, "Policies")
        # ExpirationTTL is only supported on create, not for update
        # it writes to ExpirationTime, so we need to remove that as well
        if "ExpirationTTL" in module_obj:
            del module_obj["ExpirationTTL"]
        return super().needs_update(api_obj, module_obj)


NAME_ID_SPEC = dict(
    name=dict(type="str"),
    id=dict(type="str"),
)

NODE_ID_SPEC = dict(
    node_name=dict(type="str", required=True),
    datacenter=dict(type="str", required=True),
)

SERVICE_ID_SPEC = dict(
    service_name=dict(type="str", required=True),
    datacenters=dict(type="list", elements="str"),
)

TEMPLATE_POLICY_SPEC = dict(
    template_name=dict(type="str", required=True),
    template_variables=dict(type="dict"),
)


_ARGUMENT_SPEC = {
    "description": dict(),
    "accessor_id": dict(),
    "secret_id": dict(no_log=True),
    "roles": dict(
        type="list",
        elements="dict",
        options=NAME_ID_SPEC,
        mutually_exclusive=[("name", "id")],
        required_one_of=[("name", "id")],
    ),
    "policies": dict(
        type="list",
        elements="dict",
        options=NAME_ID_SPEC,
        mutually_exclusive=[("name", "id")],
        required_one_of=[("name", "id")],
    ),
    "templated_policies": dict(
        type="list",
        elements="dict",
        options=TEMPLATE_POLICY_SPEC,
    ),
    "node_identities": dict(
        type="list",
        elements="dict",
        options=NODE_ID_SPEC,
    ),
    "service_identities": dict(
        type="list",
        elements="dict",
        options=SERVICE_ID_SPEC,
    ),
    "local": dict(type="bool"),
    "expiration_ttl": dict(type="str"),
    "state": dict(default="present", choices=["present", "absent"]),
}
_ARGUMENT_SPEC.update(AUTH_ARGUMENTS_SPEC)


def main():
    module = AnsibleModule(
        _ARGUMENT_SPEC,
        required_if=[("state", "absent", ["accessor_id"])],
        supports_check_mode=True,
    )
    consul_module = ConsulTokenModule(module)
    consul_module.execute()


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (C) 2019 Huawei
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or SPDX-License-Identifier: GPL-3.0-or-later
# https://www.gnu.org/licenses/gpl-3.0.txt)


from __future__ import annotations

###############################################################################
# Documentation
###############################################################################

DOCUMENTATION = r"""
module: hwc_vpc_peering_connect
description:
  - VPC peering management.
short_description: Creates a resource of VPC/PeeringConnect in Huawei Cloud
version_added: '0.2.0'
author: Huawei Inc. (@huaweicloud)
requirements:
  - keystoneauth1 >= 3.6.0
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  state:
    description:
      - Whether the given object should exist in Huawei Cloud.
    type: str
    choices: ['present', 'absent']
    default: 'present'
  timeouts:
    description:
      - The timeouts for each operations.
    type: dict
    default: {}
    suboptions:
      create:
        description:
          - The timeouts for create operation.
        type: str
        default: '15m'
  local_vpc_id:
    description:
      - Specifies the ID of local VPC.
    type: str
    required: true
  name:
    description:
      - Specifies the name of the VPC peering connection. The value can contain 1 to 64 characters.
    type: str
    required: true
  peering_vpc:
    description:
      - Specifies information about the peering VPC.
    type: dict
    required: true
    suboptions:
      vpc_id:
        description:
          - Specifies the ID of peering VPC.
        type: str
        required: true
      project_id:
        description:
          - Specifies the ID of the project which the peering vpc belongs to.
        type: str
  description:
    description:
      - The description of vpc peering connection.
    type: str
extends_documentation_fragment:
  - community.general._hwc
  - community.general._attributes
"""

EXAMPLES = r"""
# create a peering connect
- name: Create a local vpc
  hwc_network_vpc:
    cidr: "192.168.0.0/16"
    name: "ansible_network_vpc_test_local"
  register: vpc1
- name: Create a peering vpc
  hwc_network_vpc:
    cidr: "192.168.0.0/16"
    name: "ansible_network_vpc_test_peering"
  register: vpc2
- name: Create a peering connect
  community.general.hwc_vpc_peering_connect:
    local_vpc_id: "{{ vpc1.id }}"
    name: "ansible_network_peering_test"
    peering_vpc:
      vpc_id: "{{ vpc2.id }}"
"""

RETURN = r"""
local_vpc_id:
  description:
    - Specifies the ID of local VPC.
  type: str
  returned: success
name:
  description:
    - Specifies the name of the VPC peering connection. The value can contain 1 to 64 characters.
  type: str
  returned: success
peering_vpc:
  description:
    - Specifies information about the peering VPC.
  type: dict
  returned: success
  contains:
    vpc_id:
      description:
        - Specifies the ID of peering VPC.
      type: str
      returned: success
    project_id:
      description:
        - Specifies the ID of the project which the peering vpc belongs to.
      type: str
      returned: success
description:
  description:
    - The description of vpc peering connection.
  type: str
  returned: success
"""

from ansible_collections.community.general.plugins.module_utils._hwc_utils import (
    Config,
    HwcClientException,
    HwcClientException404,
    HwcModule,
    are_different_dicts,
    build_path,
    get_region,
    is_empty_value,
    navigate_value,
    wait_to_finish,
)


def build_module():
    return HwcModule(
        argument_spec=dict(
            state=dict(default="present", choices=["present", "absent"], type="str"),
            timeouts=dict(
                type="dict",
                options=dict(
                    create=dict(default="15m", type="str"),
                ),
                default=dict(),
            ),
            local_vpc_id=dict(type="str", required=True),
            name=dict(type="str", required=True),
            peering_vpc=dict(
                type="dict",
                required=True,
                options=dict(vpc_id=dict(type="str", required=True), project_id=dict(type="str")),
            ),
            description=dict(type="str"),
        ),
        supports_check_mode=True,
    )


def main():
    """Main function"""

    module = build_module()
    config = Config(module, "vpc")

    try:
        resource = None
        if module.params["id"]:
            resource = True
        else:
            v = search_resource(config)
            if len(v) > 1:
                raise Exception(f"Found more than one resource({', '.join([navigate_value(i, ['id']) for i in v])})")

            if len(v) == 1:
                resource = v[0]
                module.params["id"] = navigate_value(resource, ["id"])

        result = {}
        changed = False
        if module.params["state"] == "present":
            if resource is None:
                if not module.check_mode:
                    create(config)
                changed = True

            current = read_resource(config, exclude_output=True)
            expect = user_input_parameters(module)
            if are_different_dicts(expect, current):
                if not module.check_mode:
                    update(config)
                changed = True

            result = read_resource(config)
            result["id"] = module.params.get("id")
        else:
            if resource:
                if not module.check_mode:
                    delete(config)
                changed = True

    except Exception as ex:
        module.fail_json(msg=str(ex))

    else:
        result["changed"] = changed
        module.exit_json(**result)


def user_input_parameters(module):
    return {
        "description": module.params.get("description"),
        "local_vpc_id": module.params.get("local_vpc_id"),
        "name": module.params.get("name"),
        "peering_vpc": module.params.get("peering_vpc"),
    }


def create(config):
    module = config.module
    client = config.client(get_region(module), "network", "project")
    timeout = 60 * int(module.params["timeouts"]["create"].rstrip("m"))
    opts = user_input_parameters(module)

    params = build_create_parameters(opts)
    r = send_create_request(module, params, client)
    obj = async_wait_create(config, r, client, timeout)
    module.params["id"] = navigate_value(obj, ["peering", "id"])


def update(config):
    module = config.module
    client = config.client(get_region(module), "network", "project")
    opts = user_input_parameters(module)

    params = build_update_parameters(opts)
    if params:
        send_update_request(module, params, client)


def delete(config):
    module = config.module
    client = config.client(get_region(module), "network", "project")

    send_delete_request(module, None, client)

    url = build_path(module, "v2.0/vpc/peerings/{id}")

    def _refresh_status():
        try:
            client.get(url)
        except HwcClientException404:
            return True, "Done"

        except Exception:
            return None, ""

        return True, "Pending"

    timeout = 60 * int(module.params["timeouts"]["create"].rstrip("m"))
    try:
        wait_to_finish(["Done"], ["Pending"], _refresh_status, timeout)
    except Exception as ex:
        module.fail_json(msg=f"module(hwc_vpc_peering_connect): error waiting for api(delete) to be done, error= {ex}")


def read_resource(config, exclude_output=False):
    module = config.module
    client = config.client(get_region(module), "network", "project")

    res = {}

    r = send_read_request(module, client)
    res["read"] = fill_read_resp_body(r)

    return update_properties(module, res, None, exclude_output)


def _build_query_link(opts):
    query_params = []

    v = navigate_value(opts, ["local_vpc_id"])
    if v:
        query_params.append(f"vpc_id={v}")

    v = navigate_value(opts, ["name"])
    if v:
        query_params.append(f"name={v}")

    query_link = "?marker={marker}&limit=10"
    if query_params:
        query_link += f"&{'&'.join(query_params)}"

    return query_link


def search_resource(config):
    module = config.module
    client = config.client(get_region(module), "network", "project")
    opts = user_input_parameters(module)
    identity_obj = _build_identity_object(opts)
    query_link = _build_query_link(opts)
    link = f"v2.0/vpc/peerings{query_link}"

    result = []
    p = {"marker": ""}
    while True:
        url = link.format(**p)
        r = send_list_request(module, client, url)
        if not r:
            break

        for item in r:
            item = fill_list_resp_body(item)
            if not are_different_dicts(identity_obj, item):
                result.append(item)

        if len(result) > 1:
            break

        p["marker"] = r[-1].get("id")

    return result


def build_create_parameters(opts):
    params = dict()

    v = expand_create_accept_vpc_info(opts, None)
    if not is_empty_value(v):
        params["accept_vpc_info"] = v

    v = navigate_value(opts, ["description"], None)
    if not is_empty_value(v):
        params["description"] = v

    v = navigate_value(opts, ["name"], None)
    if not is_empty_value(v):
        params["name"] = v

    v = expand_create_request_vpc_info(opts, None)
    if not is_empty_value(v):
        params["request_vpc_info"] = v

    if not params:
        return params

    params = {"peering": params}

    return params


def expand_create_accept_vpc_info(d, array_index):
    r = dict()

    v = navigate_value(d, ["peering_vpc", "project_id"], array_index)
    if not is_empty_value(v):
        r["tenant_id"] = v

    v = navigate_value(d, ["peering_vpc", "vpc_id"], array_index)
    if not is_empty_value(v):
        r["vpc_id"] = v

    return r


def expand_create_request_vpc_info(d, array_index):
    r = dict()

    r["tenant_id"] = ""

    v = navigate_value(d, ["local_vpc_id"], array_index)
    if not is_empty_value(v):
        r["vpc_id"] = v

    return r


def send_create_request(module, params, client):
    url = "v2.0/vpc/peerings"
    try:
        r = client.post(url, params)
    except HwcClientException as ex:
        msg = f"module(hwc_vpc_peering_connect): error running api(create), error: {ex}"
        module.fail_json(msg=msg)

    return r


def async_wait_create(config, result, client, timeout):
    module = config.module

    path_parameters = {
        "peering_id": ["peering", "id"],
    }
    data = {key: navigate_value(result, path) for key, path in path_parameters.items()}

    url = build_path(module, "v2.0/vpc/peerings/{peering_id}", data)

    def _query_status():
        r = None
        try:
            r = client.get(url, timeout=timeout)
        except HwcClientException:
            return None, ""

        try:
            s = navigate_value(r, ["peering", "status"])
            return r, s
        except Exception:
            return None, ""

    try:
        return wait_to_finish(["ACTIVE"], ["PENDING_ACCEPTANCE"], _query_status, timeout)
    except Exception as ex:
        module.fail_json(msg=f"module(hwc_vpc_peering_connect): error waiting for api(create) to be done, error= {ex}")


def build_update_parameters(opts):
    params = dict()

    v = navigate_value(opts, ["description"], None)
    if not is_empty_value(v):
        params["description"] = v

    v = navigate_value(opts, ["name"], None)
    if not is_empty_value(v):
        params["name"] = v

    if not params:
        return params

    params = {"peering": params}

    return params


def send_update_request(module, params, client):
    url = build_path(module, "v2.0/vpc/peerings/{id}")

    try:
        r = client.put(url, params)
    except HwcClientException as ex:
        msg = f"module(hwc_vpc_peering_connect): error running api(update), error: {ex}"
        module.fail_json(msg=msg)

    return r


def send_delete_request(module, params, client):
    url = build_path(module, "v2.0/vpc/peerings/{id}")

    try:
        r = client.delete(url, params)
    except HwcClientException as ex:
        msg = f"module(hwc_vpc_peering_connect): error running api(delete), error: {ex}"
        module.fail_json(msg=msg)

    return r


def send_read_request(module, client):
    url = build_path(module, "v2.0/vpc/peerings/{id}")

    r = None
    try:
        r = client.get(url)
    except HwcClientException as ex:
        msg = f"module(hwc_vpc_peering_connect): error running api(read), error: {ex}"
        module.fail_json(msg=msg)

    return navigate_value(r, ["peering"], None)


def fill_read_resp_body(body):
    result = dict()

    v = fill_read_resp_accept_vpc_info(body.get("accept_vpc_info"))
    result["accept_vpc_info"] = v

    result["description"] = body.get("description")

    result["id"] = body.get("id")

    result["name"] = body.get("name")

    v = fill_read_resp_request_vpc_info(body.get("request_vpc_info"))
    result["request_vpc_info"] = v

    result["status"] = body.get("status")

    return result


def fill_read_resp_accept_vpc_info(value):
    if not value:
        return None

    result = dict()

    result["tenant_id"] = value.get("tenant_id")

    result["vpc_id"] = value.get("vpc_id")

    return result


def fill_read_resp_request_vpc_info(value):
    if not value:
        return None

    result = dict()

    result["tenant_id"] = value.get("tenant_id")

    result["vpc_id"] = value.get("vpc_id")

    return result


def update_properties(module, response, array_index, exclude_output=False):
    r = user_input_parameters(module)

    v = navigate_value(response, ["read", "description"], array_index)
    r["description"] = v

    v = navigate_value(response, ["read", "request_vpc_info", "vpc_id"], array_index)
    r["local_vpc_id"] = v

    v = navigate_value(response, ["read", "name"], array_index)
    r["name"] = v

    v = r.get("peering_vpc")
    v = flatten_peering_vpc(response, array_index, v, exclude_output)
    r["peering_vpc"] = v

    return r


def flatten_peering_vpc(d, array_index, current_value, exclude_output):
    result = current_value
    has_init_value = True
    if not result:
        result = dict()
        has_init_value = False

    v = navigate_value(d, ["read", "accept_vpc_info", "tenant_id"], array_index)
    result["project_id"] = v

    v = navigate_value(d, ["read", "accept_vpc_info", "vpc_id"], array_index)
    result["vpc_id"] = v

    if has_init_value:
        return result

    for v in result.values():
        if v is not None:
            return result
    return current_value


def send_list_request(module, client, url):
    r = None
    try:
        r = client.get(url)
    except HwcClientException as ex:
        msg = f"module(hwc_vpc_peering_connect): error running api(list), error: {ex}"
        module.fail_json(msg=msg)

    return navigate_value(r, ["peerings"], None)


def _build_identity_object(all_opts):
    result = dict()

    v = expand_list_accept_vpc_info(all_opts, None)
    result["accept_vpc_info"] = v

    v = navigate_value(all_opts, ["description"], None)
    result["description"] = v

    result["id"] = None

    v = navigate_value(all_opts, ["name"], None)
    result["name"] = v

    v = expand_list_request_vpc_info(all_opts, None)
    result["request_vpc_info"] = v

    result["status"] = None

    return result


def expand_list_accept_vpc_info(d, array_index):
    r = dict()

    v = navigate_value(d, ["peering_vpc", "project_id"], array_index)
    r["tenant_id"] = v

    v = navigate_value(d, ["peering_vpc", "vpc_id"], array_index)
    r["vpc_id"] = v

    for v in r.values():
        if v is not None:
            return r
    return None


def expand_list_request_vpc_info(d, array_index):
    r = dict()

    r["tenant_id"] = None

    v = navigate_value(d, ["local_vpc_id"], array_index)
    r["vpc_id"] = v

    for v in r.values():
        if v is not None:
            return r
    return None


def fill_list_resp_body(body):
    result = dict()

    v = fill_list_resp_accept_vpc_info(body.get("accept_vpc_info"))
    result["accept_vpc_info"] = v

    result["description"] = body.get("description")

    result["id"] = body.get("id")

    result["name"] = body.get("name")

    v = fill_list_resp_request_vpc_info(body.get("request_vpc_info"))
    result["request_vpc_info"] = v

    result["status"] = body.get("status")

    return result


def fill_list_resp_accept_vpc_info(value):
    if not value:
        return None

    result = dict()

    result["tenant_id"] = value.get("tenant_id")

    result["vpc_id"] = value.get("vpc_id")

    return result


def fill_list_resp_request_vpc_info(value):
    if not value:
        return None

    result = dict()

    result["tenant_id"] = value.get("tenant_id")

    result["vpc_id"] = value.get("vpc_id")

    return result


if __name__ == "__main__":
    main()



# Copyright (c) 2016-2017, Yanis Guenane <yanis+ansible@guenane.org>
# Copyright (c) 2017, Markus Teufelberger <mteufelberger+ansible@mgit.at>
# Copyright (c) 2020, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import binascii
import typing as t

from ansible.module_utils.common.text.converters import to_text

from ansible_collections.community.crypto.plugins.module_utils._crypto.cryptography_support import (
    cryptography_decode_name,
    cryptography_get_extensions_from_csr,
    cryptography_oid_to_name,
)
from ansible_collections.community.crypto.plugins.module_utils._crypto.module_backends.publickey_info import (
    get_publickey_info,
)
from ansible_collections.community.crypto.plugins.module_utils._crypto.support import (
    load_certificate_request,
)
from ansible_collections.community.crypto.plugins.module_utils._cryptography_dep import (
    COLLECTION_MINIMUM_CRYPTOGRAPHY_VERSION,
    assert_required_cryptography_version,
)

if t.TYPE_CHECKING:
    from ansible.module_utils.basic import AnsibleModule  # pragma: no cover
    from cryptography.hazmat.primitives.asymmetric.types import (  # pragma: no cover
        CertificatePublicKeyTypes,
    )

    from ansible_collections.community.crypto.plugins.plugin_utils._action_module import (  # pragma: no cover
        AnsibleActionModule,
    )
    from ansible_collections.community.crypto.plugins.plugin_utils._filter_module import (  # pragma: no cover
        FilterModuleMock,
    )

    GeneralAnsibleModule = t.Union[  # noqa: UP007
        AnsibleModule, AnsibleActionModule, FilterModuleMock
    ]  # pragma: no cover


MINIMAL_CRYPTOGRAPHY_VERSION = COLLECTION_MINIMUM_CRYPTOGRAPHY_VERSION

try:
    import cryptography
    from cryptography import x509
    from cryptography.hazmat.primitives import serialization
except ImportError:
    pass


TIMESTAMP_FORMAT = "%Y%m%d%H%M%SZ"


class CSRInfoRetrieval:
    csr: x509.CertificateSigningRequest

    def __init__(
        self, *, module: GeneralAnsibleModule, content: bytes, validate_signature: bool
    ) -> None:
        self.module = module
        self.content = content
        self.validate_signature = validate_signature
        self.name_encoding: t.Literal["ignore", "idna", "unicode"] = module.params.get(
            "name_encoding", "ignore"
        )

    def _get_subject_ordered(self) -> list[list[str]]:
        result: list[list[str]] = []
        for attribute in self.csr.subject:
            result.append(
                [cryptography_oid_to_name(attribute.oid), to_text(attribute.value)]
            )
        return result

    def _get_key_usage(self) -> tuple[list[str] | None, bool]:
        try:
            current_key_ext = self.csr.extensions.get_extension_for_class(x509.KeyUsage)
            current_key_usage = current_key_ext.value
            key_usage = {
                "digital_signature": current_key_usage.digital_signature,
                "content_commitment": current_key_usage.content_commitment,
                "key_encipherment": current_key_usage.key_encipherment,
                "data_encipherment": current_key_usage.data_encipherment,
                "key_agreement": current_key_usage.key_agreement,
                "key_cert_sign": current_key_usage.key_cert_sign,
                "crl_sign": current_key_usage.crl_sign,
                "encipher_only": False,
                "decipher_only": False,
            }
            if key_usage["key_agreement"]:
                key_usage.update(
                    {
                        "encipher_only": current_key_usage.encipher_only,
                        "decipher_only": current_key_usage.decipher_only,
                    }
                )

            key_usage_names = {
                "digital_signature": "Digital Signature",
                "content_commitment": "Non Repudiation",
                "key_encipherment": "Key Encipherment",
                "data_encipherment": "Data Encipherment",
                "key_agreement": "Key Agreement",
                "key_cert_sign": "Certificate Sign",
                "crl_sign": "CRL Sign",
                "encipher_only": "Encipher Only",
                "decipher_only": "Decipher Only",
            }
            return (
                sorted(
                    [
                        key_usage_names[name]
                        for name, value in key_usage.items()
                        if value
                    ]
                ),
                current_key_ext.critical,
            )
        except cryptography.x509.ExtensionNotFound:
            return None, False

    def _get_extended_key_usage(self) -> tuple[list[str] | None, bool]:
        try:
            ext_keyusage_ext = self.csr.extensions.get_extension_for_class(
                x509.ExtendedKeyUsage
            )
            return (
                sorted(
                    [cryptography_oid_to_name(eku) for eku in ext_keyusage_ext.value]
                ),
                ext_keyusage_ext.critical,
            )
        except cryptography.x509.ExtensionNotFound:
            return None, False

    def _get_basic_constraints(self) -> tuple[list[str] | None, bool]:
        try:
            ext_keyusage_ext = self.csr.extensions.get_extension_for_class(
                x509.BasicConstraints
            )
            result = [f"CA:{'TRUE' if ext_keyusage_ext.value.ca else 'FALSE'}"]
            if ext_keyusage_ext.value.path_length is not None:
                result.append(f"pathlen:{ext_keyusage_ext.value.path_length}")
            return sorted(result), ext_keyusage_ext.critical
        except cryptography.x509.ExtensionNotFound:
            return None, False

    def _get_ocsp_must_staple(self) -> tuple[bool | None, bool]:
        try:
            # This only works with cryptography >= 2.1
            tlsfeature_ext = self.csr.extensions.get_extension_for_class(
                x509.TLSFeature
            )
            value = (
                cryptography.x509.TLSFeatureType.status_request in tlsfeature_ext.value
            )
            return value, tlsfeature_ext.critical
        except cryptography.x509.ExtensionNotFound:
            return None, False

    def _get_subject_alt_name(self) -> tuple[list[str] | None, bool]:
        try:
            san_ext = self.csr.extensions.get_extension_for_class(
                x509.SubjectAlternativeName
            )
            result = [
                cryptography_decode_name(san, idn_rewrite=self.name_encoding)
                for san in san_ext.value
            ]
            return result, san_ext.critical
        except cryptography.x509.ExtensionNotFound:
            return None, False

    def _get_name_constraints(self) -> tuple[list[str] | None, list[str] | None, bool]:
        try:
            nc_ext = self.csr.extensions.get_extension_for_class(x509.NameConstraints)
            permitted = [
                cryptography_decode_name(san, idn_rewrite=self.name_encoding)
                for san in nc_ext.value.permitted_subtrees or []
            ]
            excluded = [
                cryptography_decode_name(san, idn_rewrite=self.name_encoding)
                for san in nc_ext.value.excluded_subtrees or []
            ]
            return permitted, excluded, nc_ext.critical
        except cryptography.x509.ExtensionNotFound:
            return None, None, False

    def _get_public_key_pem(self) -> bytes:
        return self.csr.public_key().public_bytes(
            serialization.Encoding.PEM,
            serialization.PublicFormat.SubjectPublicKeyInfo,
        )

    def _get_public_key_object(self) -> CertificatePublicKeyTypes:
        return self.csr.public_key()

    def _get_subject_key_identifier(self) -> bytes | None:
        try:
            ext = self.csr.extensions.get_extension_for_class(x509.SubjectKeyIdentifier)
            return ext.value.digest
        except cryptography.x509.ExtensionNotFound:
            return None

    def _get_authority_key_identifier(
        self,
    ) -> tuple[bytes | None, list[str] | None, int | None]:
        try:
            ext = self.csr.extensions.get_extension_for_class(
                x509.AuthorityKeyIdentifier
            )
            issuer = None
            if ext.value.authority_cert_issuer is not None:
                issuer = [
                    cryptography_decode_name(san, idn_rewrite=self.name_encoding)
                    for san in ext.value.authority_cert_issuer
                ]
            return (
                ext.value.key_identifier,
                issuer,
                ext.value.authority_cert_serial_number,
            )
        except cryptography.x509.ExtensionNotFound:
            return None, None, None

    def _get_all_extensions(self) -> dict[str, dict[str, bool | str]]:
        return cryptography_get_extensions_from_csr(self.csr)

    def _is_signature_valid(self) -> bool:
        return self.csr.is_signature_valid

    def get_info(self, *, prefer_one_fingerprint: bool = False) -> dict[str, t.Any]:
        result: dict[str, t.Any] = {}
        self.csr = load_certificate_request(
            content=self.content,
        )

        subject = self._get_subject_ordered()
        result["subject"] = {}
        for k, v in subject:
            result["subject"][k] = v
        result["subject_ordered"] = subject
        result["key_usage"], result["key_usage_critical"] = self._get_key_usage()
        result["extended_key_usage"], result["extended_key_usage_critical"] = (
            self._get_extended_key_usage()
        )
        result["basic_constraints"], result["basic_constraints_critical"] = (
            self._get_basic_constraints()
        )
        result["ocsp_must_staple"], result["ocsp_must_staple_critical"] = (
            self._get_ocsp_must_staple()
        )
        result["subject_alt_name"], result["subject_alt_name_critical"] = (
            self._get_subject_alt_name()
        )
        (
            result["name_constraints_permitted"],
            result["name_constraints_excluded"],
            result["name_constraints_critical"],
        ) = self._get_name_constraints()

        result["public_key"] = to_text(self._get_public_key_pem())

        public_key_info = get_publickey_info(
            module=self.module,
            key=self._get_public_key_object(),
            prefer_one_fingerprint=prefer_one_fingerprint,
        )
        result.update(
            {
                "public_key_type": public_key_info["type"],
                "public_key_data": public_key_info["public_data"],
                "public_key_fingerprints": public_key_info["fingerprints"],
            }
        )

        ski_bytes = self._get_subject_key_identifier()
        ski = None
        if ski_bytes is not None:
            ski = binascii.hexlify(ski_bytes).decode("ascii")
            ski = ":".join([ski[i : i + 2] for i in range(0, len(ski), 2)])
        result["subject_key_identifier"] = ski

        aki_bytes, aci, acsn = self._get_authority_key_identifier()
        aki = None
        if aki_bytes is not None:
            aki = binascii.hexlify(aki_bytes).decode("ascii")
            aki = ":".join([aki[i : i + 2] for i in range(0, len(aki), 2)])
        result["authority_key_identifier"] = aki
        result["authority_cert_issuer"] = aci
        result["authority_cert_serial_number"] = acsn

        result["extensions_by_oid"] = self._get_all_extensions()

        result["signature_valid"] = self._is_signature_valid()
        if self.validate_signature and not result["signature_valid"]:
            self.module.fail_json(msg="CSR signature is invalid!", **result)
        return result


def get_csr_info(
    *,
    module: GeneralAnsibleModule,
    content: bytes,
    validate_signature: bool = True,
    prefer_one_fingerprint: bool = False,
) -> dict[str, t.Any]:
    info = CSRInfoRetrieval(
        module=module, content=content, validate_signature=validate_signature
    )
    return info.get_info(prefer_one_fingerprint=prefer_one_fingerprint)


def select_backend(
    *, module: GeneralAnsibleModule, content: bytes, validate_signature: bool = True
) -> CSRInfoRetrieval:
    assert_required_cryptography_version(
        module, minimum_cryptography_version=MINIMAL_CRYPTOGRAPHY_VERSION
    )
    return CSRInfoRetrieval(
        module=module, content=content, validate_signature=validate_signature
    )


__all__ = ("CSRInfoRetrieval", "get_csr_info", "select_backend")



# Copyright (c) Ansible project
# Simplified BSD License (see LICENSES/BSD-2-Clause.txt or https://opensource.org/licenses/BSD-2-Clause)
# SPDX-License-Identifier: BSD-2-Clause

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import os
import re
import typing as t

if t.TYPE_CHECKING:
    from ansible.module_utils.basic import AnsibleModule


def _create_regex_group_complement(s: str) -> re.Pattern:
    lines = (line.strip() for line in s.split("\n") if line.strip())
    chars = [_f for _f in (line.split("#")[0].strip() for line in lines) if _f]
    group = rf"[^{''.join(chars)}]"
    return re.compile(group)


class HomebrewValidate:
    # class regexes ------------------------------------------------ {{{
    VALID_PATH_CHARS = rf"""
        \w                  # alphanumeric characters (i.e., [a-zA-Z0-9_])
        \s                  # spaces
        :                   # colons
        {os.path.sep}       # the OS-specific path separator
        .                   # dots
        \-                  # dashes
    """

    VALID_BREW_PATH_CHARS = rf"""
        \w                  # alphanumeric characters (i.e., [a-zA-Z0-9_])
        \s                  # spaces
        {os.path.sep}       # the OS-specific path separator
        .                   # dots
        \-                  # dashes
    """

    VALID_PACKAGE_CHARS = r"""
        \w                  # alphanumeric characters (i.e., [a-zA-Z0-9_])
        .                   # dots
        /                   # slash (for taps)
        \+                  # plusses
        \-                  # dashes
        :                   # colons (for URLs)
        @                   # at-sign
    """

    INVALID_PATH_REGEX = _create_regex_group_complement(VALID_PATH_CHARS)
    INVALID_BREW_PATH_REGEX = _create_regex_group_complement(VALID_BREW_PATH_CHARS)
    INVALID_PACKAGE_REGEX = _create_regex_group_complement(VALID_PACKAGE_CHARS)
    # /class regexes ----------------------------------------------- }}}

    # class validations -------------------------------------------- {{{
    @classmethod
    def valid_path(cls, path: list[str] | str) -> bool:
        """
        `path` must be one of:
         - list of paths
         - a string containing only:
             - alphanumeric characters
             - dashes
             - dots
             - spaces
             - colons
             - os.path.sep
        """

        if isinstance(path, str):
            return not cls.INVALID_PATH_REGEX.search(path)

        try:
            iter(path)
        except TypeError:
            return False
        else:
            paths = path
            return all(cls.valid_brew_path(path_) for path_ in paths)

    @classmethod
    def valid_brew_path(cls, brew_path: str | None) -> bool:
        """
        `brew_path` must be one of:
         - None
         - a string containing only:
             - alphanumeric characters
             - dashes
             - dots
             - spaces
             - os.path.sep
        """

        if brew_path is None:
            return True

        return isinstance(brew_path, str) and not cls.INVALID_BREW_PATH_REGEX.search(brew_path)

    @classmethod
    def valid_package(cls, package: str | None) -> bool:
        """A valid package is either None or alphanumeric."""

        if package is None:
            return True

        return isinstance(package, str) and not cls.INVALID_PACKAGE_REGEX.search(package)


def parse_brew_path(module: AnsibleModule) -> str:
    """Attempt to find the Homebrew executable path.

    Requires:
        - module has a `path` parameter
        - path is a valid path string for the target OS. Otherwise, module.fail_json()
          is called with msg="Invalid_path: <path>".
    """
    path = module.params["path"]
    if not HomebrewValidate.valid_path(path):
        module.fail_json(msg=f"Invalid path: {path}")

    if isinstance(path, str):
        paths = path.split(":")
    elif isinstance(path, list):
        paths = path
    else:
        module.fail_json(msg=f"Invalid path: {path}")

    brew_path = module.get_bin_path("brew", required=True, opt_dirs=paths)
    if not HomebrewValidate.valid_brew_path(brew_path):
        module.fail_json(msg=f"Invalid brew path: {brew_path}")

    return brew_path



#!/usr/bin/python

# Copyright (c) 2023, Ondrej Zvara (ozvara1@gmail.com)
# Based on code:
# Copyright (c) 2021, Lennert Mertens (lennert@nubera.be)
# Copyright (c) 2021, Werner Dijkerman (ikben@werner-dijkerman.nl)
# Copyright (c) 2015, Werner Dijkerman (ikben@werner-dijkerman.nl)
# Copyright (c) 2019, Guillaume Martinez (lunik@tiwabbit.fr)
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later
from __future__ import annotations

DOCUMENTATION = r"""
module: gitlab_merge_request
short_description: Create, update, or delete GitLab merge requests
version_added: 7.1.0
description:
  - Creates a merge request if it does not exist.
  - When a single merge request does exist, it is updated if the provided parameters are different.
  - When a single merge request does exist and O(state=absent), the merge request is deleted.
  - When multiple merge requests are detected, the task fails.
  - Existing merge requests are matched based on O(title), O(source_branch), O(target_branch), and O(state_filter) filters.
author:
  - zvaraondrej (@zvaraondrej)
requirements:
  - python-gitlab >= 2.3.0
extends_documentation_fragment:
  - community.general._auth_basic
  - community.general._gitlab
  - community.general._attributes

attributes:
  check_mode:
    support: full
  diff_mode:
    support: none

options:
  state:
    description:
      - Create or delete merge request.
    default: present
    type: str
    choices: ["present", "absent"]
  project:
    description:
      - The path or name of the project.
    required: true
    type: str
  source_branch:
    description:
      - Merge request's source branch.
      - Ignored while updating existing merge request.
    required: true
    type: str
  target_branch:
    description:
      - Merge request's target branch.
    required: true
    type: str
  title:
    description:
      - A title for the merge request.
    type: str
    required: true
  description:
    description:
      - A description for the merge request.
      - Gets overridden by a content of file specified at O(description_path), if found.
    type: str
  description_path:
    description:
      - A path of file containing merge request's description.
      - Accepts MarkDown formatted files.
    type: path
  labels:
    description:
      - Comma separated list of label names.
    type: str
    default: ""
  remove_source_branch:
    description:
      - Flag indicating if a merge request should remove the source branch when merging.
    type: bool
    default: false
  state_filter:
    description:
      - Filter specifying state of merge requests while searching.
    type: str
    choices: ["opened", "closed", "locked", "merged"]
    default: opened
  assignee_ids:
    description:
      - Comma separated list of assignees usernames omitting V(@) character.
      - Set to empty string to unassign all assignees.
    type: str
  reviewer_ids:
    description:
      - Comma separated list of reviewers usernames omitting V(@) character.
      - Set to empty string to unassign all reviewers.
    type: str
"""


EXAMPLES = r"""
- name: Create Merge Request from branch1 to branch2
  community.general.gitlab_merge_request:
    api_url: https://gitlab.com
    api_token: secret_access_token
    project: "group1/project1"
    source_branch: branch1
    target_branch: branch2
    title: "Ansible demo MR"
    description: "Demo MR description"
    labels: "Ansible,Demo"
    state_filter: "opened"
    remove_source_branch: true
    state: present

- name: Delete Merge Request from branch1 to branch2
  community.general.gitlab_merge_request:
    api_url: https://gitlab.com
    api_token: secret_access_token
    project: "group1/project1"
    source_branch: branch1
    target_branch: branch2
    title: "Ansible demo MR"
    state_filter: "opened"
    state: absent
"""

RETURN = r"""
msg:
  description: Success or failure message.
  returned: always
  type: str
  sample: "Success"

mr:
  description: API object.
  returned: success
  type: dict
"""

from ansible.module_utils.api import basic_auth_argument_spec
from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.common.text.converters import to_text

from ansible_collections.community.general.plugins.module_utils._gitlab import (
    auth_argument_spec,
    find_project,
    gitlab,
    gitlab_authentication,
)
from ansible_collections.community.general.plugins.module_utils._version import LooseVersion


class GitlabMergeRequest:
    def __init__(self, module, project, gitlab_instance):
        self._gitlab = gitlab_instance
        self._module = module
        self.project = project

    """
    @param branch Name of the branch
    """

    def get_branch(self, branch):
        try:
            return self.project.branches.get(branch)
        except gitlab.exceptions.GitlabGetError as e:
            self._module.fail_json(msg=f"Failed to get the branch: {e}")

    """
    @param title Title of the Merge Request
    @param source_branch Merge Request's source branch
    @param target_branch Merge Request's target branch
    @param state_filter Merge Request's state to filter on
    """

    def get_mr(self, title, source_branch, target_branch, state_filter):
        mrs = []
        try:
            mrs = self.project.mergerequests.list(
                search=title, source_branch=source_branch, target_branch=target_branch, state=state_filter
            )
        except gitlab.exceptions.GitlabGetError as e:
            self._module.fail_json(msg=f"Failed to list the Merge Request: {e}")

        if len(mrs) > 1:
            self._module.fail_json(msg="Multiple Merge Requests matched search criteria.")
        if len(mrs) == 1:
            try:
                return self.project.mergerequests.get(id=mrs[0].iid)
            except gitlab.exceptions.GitlabGetError as e:
                self._module.fail_json(msg=f"Failed to get the Merge Request: {e}")

    """
    @param username Name of the user
    """

    def get_user(self, username):
        users = []
        try:
            users = [user for user in self.project.users.list(username=username, all=True) if user.username == username]
        except gitlab.exceptions.GitlabGetError as e:
            self._module.fail_json(msg=f"Failed to list the users: {e}")

        if len(users) > 1:
            self._module.fail_json(msg="Multiple Users matched search criteria.")
        elif len(users) < 1:
            self._module.fail_json(msg="No User matched search criteria.")
        else:
            return users[0]

    """
    @param users List of usernames
    """

    def get_user_ids(self, users):
        return [self.get_user(user).id for user in users]

    """
    @param options Options of the Merge Request
    """

    def create_mr(self, options):
        if self._module.check_mode:
            self._module.exit_json(changed=True, msg=f"Successfully created the Merge Request {options['title']}")

        try:
            return self.project.mergerequests.create(options)
        except gitlab.exceptions.GitlabCreateError as e:
            self._module.fail_json(msg=f"Failed to create Merge Request: {e}")

    """
    @param mr Merge Request object to delete
    """

    def delete_mr(self, mr):
        if self._module.check_mode:
            self._module.exit_json(changed=True, msg=f"Successfully deleted the Merge Request {mr['title']}")

        try:
            return mr.delete()
        except gitlab.exceptions.GitlabDeleteError as e:
            self._module.fail_json(msg=f"Failed to delete Merge Request: {e}")

    """
    @param mr Merge Request object to update
    """

    def update_mr(self, mr, options):
        if self._module.check_mode:
            self._module.exit_json(changed=True, msg=f"Successfully updated the Merge Request {mr['title']}")

        try:
            return self.project.mergerequests.update(mr.iid, options)
        except gitlab.exceptions.GitlabUpdateError as e:
            self._module.fail_json(msg=f"Failed to update Merge Request: {e}")

    """
    @param mr Merge Request object to evaluate
    @param options New options to update MR with
    """

    def mr_has_changed(self, mr, options):
        for key, value in options.items():
            if value is not None:
                # see https://gitlab.com/gitlab-org/gitlab-foss/-/issues/27355
                if key == "remove_source_branch":
                    key = "force_remove_source_branch"

                if key == "assignee_ids":
                    if value != sorted([user["id"] for user in mr.assignees]):
                        return True

                elif key == "reviewer_ids":
                    if value != sorted([user["id"] for user in mr.reviewers]):
                        return True

                elif key == "labels":
                    if value != sorted(getattr(mr, key)):
                        return True

                elif getattr(mr, key) != value:
                    return True

        return False


def main():
    argument_spec = basic_auth_argument_spec()
    argument_spec.update(auth_argument_spec())
    argument_spec.update(
        project=dict(type="str", required=True),
        source_branch=dict(type="str", required=True),
        target_branch=dict(type="str", required=True),
        title=dict(type="str", required=True),
        description=dict(type="str"),
        labels=dict(type="str", default=""),
        description_path=dict(type="path"),
        remove_source_branch=dict(type="bool", default=False),
        state_filter=dict(type="str", default="opened", choices=["opened", "closed", "locked", "merged"]),
        assignee_ids=dict(type="str"),
        reviewer_ids=dict(type="str"),
        state=dict(type="str", default="present", choices=["absent", "present"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        mutually_exclusive=[
            ["api_username", "api_token"],
            ["api_username", "api_oauth_token"],
            ["api_username", "api_job_token"],
            ["api_token", "api_oauth_token"],
            ["api_token", "api_job_token"],
            ["description", "description_path"],
        ],
        required_together=[
            ["api_username", "api_password"],
        ],
        required_one_of=[["api_username", "api_token", "api_oauth_token", "api_job_token"]],
        required_if=[
            ["state", "present", ["source_branch", "target_branch", "title"], True],
            ["state", "absent", ["source_branch", "target_branch", "title"], True],
        ],
        supports_check_mode=True,
    )

    # check prerequisites and connect to gitlab server
    gitlab_instance = gitlab_authentication(module)

    project = module.params["project"]
    source_branch = module.params["source_branch"]
    target_branch = module.params["target_branch"]
    title = module.params["title"]
    description = module.params["description"]
    labels = module.params["labels"]
    description_path = module.params["description_path"]
    remove_source_branch = module.params["remove_source_branch"]
    state_filter = module.params["state_filter"]
    assignee_ids = module.params["assignee_ids"]
    reviewer_ids = module.params["reviewer_ids"]
    state = module.params["state"]

    gitlab_version = gitlab.__version__
    if LooseVersion(gitlab_version) < LooseVersion("2.3.0"):
        module.fail_json(
            msg=f"community.general.gitlab_merge_request requires python-gitlab Python module >= 2.3.0 (installed version: [{gitlab_version}])."
            " Please upgrade python-gitlab to version 2.3.0 or above."
        )

    this_project = find_project(gitlab_instance, project)
    if this_project is None:
        module.fail_json(msg=f"Failed to get the project: {project}")

    this_gitlab = GitlabMergeRequest(module=module, project=this_project, gitlab_instance=gitlab_instance)

    r_source_branch = this_gitlab.get_branch(source_branch)
    if not r_source_branch:
        module.fail_json(msg=f"Source branch {r_source_branch} not exist.")

    r_target_branch = this_gitlab.get_branch(target_branch)
    if not r_target_branch:
        module.fail_json(msg=f"Destination branch {r_target_branch} not exist.")

    this_mr = this_gitlab.get_mr(title, source_branch, target_branch, state_filter)

    if state == "present":
        if description_path:
            try:
                with open(description_path, "rb") as f:
                    description = to_text(f.read(), errors="surrogate_or_strict")
            except OSError as e:
                module.fail_json(msg=f"Cannot open {description_path}: {e}")

        # sorting necessary in order to properly detect changes, as we don't want to get false positive
        # results due to differences in ids ordering; see `mr_has_changed()`
        assignee_ids = sorted(this_gitlab.get_user_ids(assignee_ids.split(","))) if assignee_ids else []
        reviewer_ids = sorted(this_gitlab.get_user_ids(reviewer_ids.split(","))) if reviewer_ids else []
        labels = sorted(labels.split(",")) if labels else []

        options = {
            "target_branch": target_branch,
            "title": title,
            "description": description,
            "labels": labels,
            "remove_source_branch": remove_source_branch,
            "reviewer_ids": reviewer_ids,
            "assignee_ids": assignee_ids,
        }

        if not this_mr:
            options["source_branch"] = source_branch

            mr = this_gitlab.create_mr(options)
            module.exit_json(
                changed=True,
                msg=f"Created the Merge Request {title} from branch {source_branch} to branch {target_branch}.",
                mr=mr.asdict(),
            )
        else:
            if this_gitlab.mr_has_changed(this_mr, options):
                mr = this_gitlab.update_mr(this_mr, options)
                module.exit_json(
                    changed=True,
                    msg=f"Merge Request {title} from branch {source_branch} to branch {target_branch} updated.",
                    mr=mr,
                )
            else:
                module.exit_json(
                    changed=False,
                    msg=f"Merge Request {title} from branch {source_branch} to branch {target_branch} already exist",
                    mr=this_mr.asdict(),
                )
    elif this_mr and state == "absent":
        mr = this_gitlab.delete_mr(this_mr)
        module.exit_json(
            changed=True,
            msg=f"Merge Request {title} from branch {source_branch} to branch {target_branch} deleted.",
            mr=mr,
        )
    else:
        module.exit_json(changed=False, msg="No changes are needed.", mr=this_mr.asdict())


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright Ansible Project
#
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: dnsimple
short_description: Interface with dnsimple.com (a DNS hosting service)
description:
  - 'Manages domains and records using the DNSimple API, see the docs: U(http://developer.dnsimple.com/).'
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  account_email:
    description:
      - Account email. If omitted, the environment variables E(DNSIMPLE_EMAIL) and E(DNSIMPLE_API_TOKEN) are looked for.
      - 'If those variables are not found, a C(.dnsimple) file is looked for, see: U(https://github.com/mikemaccana/dnsimple-python#getting-started).'
      - C(.dnsimple) config files are only supported in dnsimple-python<2.0.0.
    type: str
  account_api_token:
    description:
      - Account API token. See O(account_email) for more information.
    type: str
  domain:
    description:
      - Domain to work with. Can be the domain name (for example V(mydomain.com)) or the numeric ID of the domain in DNSimple.
      - If omitted, a list of domains is returned.
      - If domain is present but the domain does not exist, it is created.
    type: str
  record:
    description:
      - Record to add, if blank a record for the domain is created, supports the wildcard (*).
    type: str
  record_ids:
    description:
      - List of records to ensure they either exist or do not exist.
    type: list
    elements: str
  type:
    description:
      - The type of DNS record to create.
    choices:
      - A
      - ALIAS
      - CNAME
      - MX
      - SPF
      - URL
      - TXT
      - NS
      - SRV
      - NAPTR
      - PTR
      - AAAA
      - SSHFP
      - HINFO
      - POOL
      - CAA
    type: str
  ttl:
    description:
      - The TTL to give the new record in seconds.
    default: 3600
    type: int
  value:
    description:
      - Record value.
      - Must be specified when trying to ensure a record exists.
    type: str
  priority:
    description:
      - Record priority.
    type: int
  state:
    description:
      - Whether the record should exist or not.
    choices: ['present', 'absent']
    default: present
    type: str
  solo:
    description:
      - Whether the record should be the only one for that record type and record name.
      - Only use with O(state) is set to V(present) on a record.
    type: 'bool'
    default: false
  sandbox:
    description:
      - Use the DNSimple sandbox environment.
      - Requires a dedicated account in the dnsimple sandbox environment.
      - Check U(https://developer.dnsimple.com/sandbox/) for more information.
    type: 'bool'
    default: false
    version_added: 3.5.0
requirements:
  - "dnsimple >= 2.0.0"
author: "Alex Coomans (@drcapulet)"
"""

EXAMPLES = r"""
- name: Authenticate using email and API token and fetch all domains
  community.general.dnsimple:
    account_email: test@example.com
    account_api_token: dummyapitoken
  delegate_to: localhost

- name: Delete a domain
  community.general.dnsimple:
    domain: my.com
    state: absent
  delegate_to: localhost

- name: Create a test.my.com A record to point to 127.0.0.1
  community.general.dnsimple:
    domain: my.com
    record: test
    type: A
    value: 127.0.0.1
  delegate_to: localhost
  register: record

- name: Delete record using record_ids
  community.general.dnsimple:
    domain: my.com
    record_ids: '{{ record["id"] }}'
    state: absent
  delegate_to: localhost

- name: Create a my.com CNAME record to example.com
  community.general.dnsimple:
    domain: my.com
    record: ''
    type: CNAME
    value: example.com
    state: present
  delegate_to: localhost

- name: Change TTL value for a record
  community.general.dnsimple:
    domain: my.com
    record: ''
    type: CNAME
    value: example.com
    ttl: 600
    state: present
  delegate_to: localhost

- name: Delete the record
  community.general.dnsimple:
    domain: my.com
    record: ''
    type: CNAME
    value: example.com
    state: absent
  delegate_to: localhost
"""

RETURN = r"""#"""

import re
import traceback

from ansible_collections.community.general.plugins.module_utils._version import LooseVersion


class DNSimpleV2:
    """class which uses dnsimple-python >= 2"""

    def __init__(self, account_email, account_api_token, sandbox, module):
        """init"""
        self.module = module
        self.account_email = account_email
        self.account_api_token = account_api_token
        self.sandbox = sandbox
        self.pagination_per_page = 30
        self.dnsimple_client()
        self.dnsimple_account()

    def dnsimple_client(self):
        """creates a dnsimple client object"""
        if self.account_email and self.account_api_token:
            client = Client(
                sandbox=self.sandbox,
                email=self.account_email,
                access_token=self.account_api_token,
                user_agent="ansible/community.general",
            )
        else:
            msg = (
                "Option account_email or account_api_token not provided. "
                "Dnsimple authentication with a .dnsimple config file is not "
                "supported with dnsimple-python>=2.0.0"
            )
            raise DNSimpleException(msg)
        client.identity.whoami()
        self.client = client

    def dnsimple_account(self):
        """select a dnsimple account. If a user token is used for authentication,
        this user must only have access to a single account"""
        account = self.client.identity.whoami().data.account
        # user supplied a user token instead of account api token
        if not account:
            accounts = Accounts(self.client).list_accounts().data
            if len(accounts) != 1:
                msg = (
                    "The provided dnsimple token is a user token with multiple accounts."
                    "Use an account token or a user token with access to a single account."
                    "See https://support.dnsimple.com/articles/api-access-token/"
                )
                raise DNSimpleException(msg)
            account = accounts[0]
        self.account = account

    def get_all_domains(self):
        """returns a list of all domains"""
        domain_list = self._get_paginated_result(self.client.domains.list_domains, account_id=self.account.id)
        return [d.__dict__ for d in domain_list]

    def get_domain(self, domain):
        """returns a single domain by name or id"""
        try:
            dr = self.client.domains.get_domain(self.account.id, domain).data.__dict__
        except DNSimpleException as e:
            exception_string = str(e.message)
            if re.match(r"^Domain .+ not found$", exception_string):
                dr = None
            else:
                raise
        return dr

    def create_domain(self, domain):
        """create a single domain"""
        return self.client.domains.create_domain(self.account.id, domain).data.__dict__

    def delete_domain(self, domain):
        """delete a single domain"""
        self.client.domains.delete_domain(self.account.id, domain)

    def get_records(self, zone, dnsimple_filter=None):
        """return dns resource records which match a specified filter"""
        records_list = self._get_paginated_result(
            self.client.zones.list_records, account_id=self.account.id, zone=zone, filter=dnsimple_filter
        )
        return [d.__dict__ for d in records_list]

    def delete_record(self, domain, rid):
        """delete a single dns resource record"""
        self.client.zones.delete_record(self.account.id, domain, rid)

    def update_record(self, domain, rid, ttl=None, priority=None):
        """update a single dns resource record"""
        zr = ZoneRecordUpdateInput(ttl=ttl, priority=priority)
        result = self.client.zones.update_record(self.account.id, str(domain), str(rid), zr).data.__dict__
        return result

    def create_record(self, domain, name, record_type, content, ttl=None, priority=None):
        """create a single dns resource record"""
        zr = ZoneRecordInput(name=name, type=record_type, content=content, ttl=ttl, priority=priority)
        return self.client.zones.create_record(self.account.id, str(domain), zr).data.__dict__

    def _get_paginated_result(self, operation, **options):
        """return all results of a paginated api response"""
        records_pagination = operation(per_page=self.pagination_per_page, **options).pagination
        result_list = []
        for page in range(1, records_pagination.total_pages + 1):
            page_data = operation(per_page=self.pagination_per_page, page=page, **options).data
            result_list.extend(page_data)
        return result_list


DNSIMPLE_IMP_ERR = []
HAS_DNSIMPLE = False
try:
    # try to import dnsimple >= 2.0.0
    from dnsimple import Client, DNSimpleException
    from dnsimple.service import Accounts
    from dnsimple.struct.zone_record import ZoneRecordInput, ZoneRecordUpdateInput
    from dnsimple.version import version as dnsimple_version

    HAS_DNSIMPLE = True
except ImportError:
    DNSIMPLE_IMP_ERR.append(traceback.format_exc())

from ansible.module_utils.basic import AnsibleModule, env_fallback, missing_required_lib


def main():
    module = AnsibleModule(
        argument_spec=dict(
            account_email=dict(type="str", fallback=(env_fallback, ["DNSIMPLE_EMAIL"])),
            account_api_token=dict(type="str", no_log=True, fallback=(env_fallback, ["DNSIMPLE_API_TOKEN"])),
            domain=dict(type="str"),
            record=dict(type="str"),
            record_ids=dict(type="list", elements="str"),
            type=dict(
                type="str",
                choices=[
                    "A",
                    "ALIAS",
                    "CNAME",
                    "MX",
                    "SPF",
                    "URL",
                    "TXT",
                    "NS",
                    "SRV",
                    "NAPTR",
                    "PTR",
                    "AAAA",
                    "SSHFP",
                    "HINFO",
                    "POOL",
                    "CAA",
                ],
            ),
            ttl=dict(type="int", default=3600),
            value=dict(type="str"),
            priority=dict(type="int"),
            state=dict(type="str", choices=["present", "absent"], default="present"),
            solo=dict(type="bool", default=False),
            sandbox=dict(type="bool", default=False),
        ),
        required_together=[["record", "value"]],
        supports_check_mode=True,
    )

    if not HAS_DNSIMPLE:
        module.fail_json(msg=missing_required_lib("dnsimple"), exception=DNSIMPLE_IMP_ERR[0])

    account_email = module.params.get("account_email")
    account_api_token = module.params.get("account_api_token")
    domain = module.params.get("domain")
    record = module.params.get("record")
    record_ids = module.params.get("record_ids")
    record_type = module.params.get("type")
    ttl = module.params.get("ttl")
    value = module.params.get("value")
    priority = module.params.get("priority")
    state = module.params.get("state")
    is_solo = module.params.get("solo")
    sandbox = module.params.get("sandbox")

    DNSIMPLE_MAJOR_VERSION = LooseVersion(dnsimple_version).version[0]

    try:
        if DNSIMPLE_MAJOR_VERSION < 2:
            module.fail_json(
                msg="Support for python-dnsimple < 2 has been removed in community.general 5.0.0. Update python-dnsimple to version >= 2.0.0."
            )
        ds = DNSimpleV2(account_email, account_api_token, sandbox, module)
        # Let's figure out what operation we want to do
        # No domain, return a list
        if not domain:
            all_domains = ds.get_all_domains()
            module.exit_json(changed=False, result=all_domains)

        # Domain & No record
        if record is None and not record_ids:
            if domain.isdigit():
                typed_domain = int(domain)
            else:
                typed_domain = str(domain)
            dr = ds.get_domain(typed_domain)
            # domain does not exist
            if state == "present":
                if dr:
                    module.exit_json(changed=False, result=dr)
                else:
                    if module.check_mode:
                        module.exit_json(changed=True)
                    else:
                        response = ds.create_domain(domain)
                        module.exit_json(changed=True, result=response)
            # state is absent
            else:
                if dr:
                    if not module.check_mode:
                        ds.delete_domain(domain)
                    module.exit_json(changed=True)
                else:
                    module.exit_json(changed=False)

        # need the not none check since record could be an empty string
        if record is not None:
            if not record_type:
                module.fail_json(msg="Missing the record type")
            if not value:
                module.fail_json(msg="Missing the record value")

            records_list = ds.get_records(domain, dnsimple_filter={"name": record})
            rr = next(
                (r for r in records_list if r["name"] == record and r["type"] == record_type and r["content"] == value),
                None,
            )
            if state == "present":
                changed = False
                if is_solo:
                    # delete any records that have the same name and record type
                    same_type = [r["id"] for r in records_list if r["name"] == record and r["type"] == record_type]
                    if rr:
                        same_type = [rid for rid in same_type if rid != rr["id"]]
                    if same_type:
                        if not module.check_mode:
                            for rid in same_type:
                                ds.delete_record(domain, rid)
                        changed = True
                if rr:
                    # check if we need to update
                    if rr["ttl"] != ttl or rr["priority"] != priority:
                        if module.check_mode:
                            module.exit_json(changed=True)
                        else:
                            response = ds.update_record(domain, rr["id"], ttl, priority)
                            module.exit_json(changed=True, result=response)
                    else:
                        module.exit_json(changed=changed, result=rr)
                else:
                    # create it
                    if module.check_mode:
                        module.exit_json(changed=True)
                    else:
                        response = ds.create_record(domain, record, record_type, value, ttl, priority)
                        module.exit_json(changed=True, result=response)
            # state is absent
            else:
                if rr:
                    if not module.check_mode:
                        ds.delete_record(domain, rr["id"])
                    module.exit_json(changed=True)
                else:
                    module.exit_json(changed=False)

        # Make sure these record_ids either all exist or none
        if record_ids:
            current_records = ds.get_records(domain, dnsimple_filter=None)
            current_record_ids = [str(d["id"]) for d in current_records]
            wanted_record_ids = [str(r) for r in record_ids]
            if state == "present":
                difference = list(set(wanted_record_ids) - set(current_record_ids))
                if difference:
                    module.fail_json(msg=f"Missing the following records: {difference}")
                else:
                    module.exit_json(changed=False)
            # state is absent
            else:
                difference = list(set(wanted_record_ids) & set(current_record_ids))
                if difference:
                    if not module.check_mode:
                        for rid in difference:
                            ds.delete_record(domain, rid)
                    module.exit_json(changed=True)
                else:
                    module.exit_json(changed=False)

    except DNSimpleException as e:
        if DNSIMPLE_MAJOR_VERSION > 1:
            module.fail_json(msg=f"DNSimple exception: {e.message}")
        else:
            module.fail_json(msg=f"DNSimple exception: {e.args[0]['message']}")
    module.fail_json(msg="Unknown what you wanted me to do")


if __name__ == "__main__":
    main()



# Copyright (c) 2025 Felix Fontein
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import typing as t

from ansible_collections.community.dns.plugins.module_utils._record import (
    RecordIDT,
    RecordIDT_co,
)
from ansible_collections.community.dns.plugins.module_utils._record import (
    format_ttl as _format_ttl,
)

if t.TYPE_CHECKING:
    from collections.abc import Mapping, Sequence  # pragma: no cover

    from ._conversion.converter import RecordConverter  # pragma: no cover
    from ._record import DNSRecord, IDNSRecord  # pragma: no cover


RecordSetIDT = t.TypeVar("RecordSetIDT")
RecordSetIDT_co = t.TypeVar("RecordSetIDT_co", covariant=True)


class IDNSRecordSet(t.Protocol, t.Generic[RecordSetIDT_co, RecordIDT_co]):
    @property
    def id(self) -> RecordSetIDT_co: ...

    @property
    def type(self) -> str: ...

    @property
    def prefix(self) -> str | None: ...

    @property
    def ttl(self) -> int | None: ...

    @property
    def records(self) -> Sequence[IDNSRecord[RecordIDT_co]]: ...

    @property
    def extra(self) -> Mapping[str, str]: ...


class DNSRecordSet(t.Generic[RecordSetIDT, RecordIDT]):
    def __init__(self, *, record_set_id: RecordSetIDT, record_type: str) -> None:
        self.id = record_set_id
        self.type = record_type
        self.prefix: str | None = None
        self.ttl: int | None = None
        self.records: list[DNSRecord] = []
        self.extra: dict[str, str] = {}

    def clone(self) -> DNSRecordSet[RecordSetIDT, RecordIDT]:
        result: DNSRecordSet[RecordSetIDT, RecordIDT] = DNSRecordSet(
            record_set_id=self.id, record_type=self.type
        )
        result.prefix = self.prefix
        result.records = [record.clone() for record in self.records]
        result.ttl = self.ttl
        result.extra = dict(self.extra)
        return result

    def __str__(self) -> str:
        data = []
        if self.id is not None:
            data.append(f"id: {self.id}")
        data.append(f"type: {self.type}")
        if self.prefix is not None:
            data.append(f"prefix: {self.prefix!r}")
        else:
            data.append("prefix: (none)")
        data.append(f"ttl: {_format_ttl(self.ttl)}")
        data.append(f"records: [{', '.join([str(record) for record in self.records])}]")
        if self.extra:
            data.append(f"extra: {self.extra}")
        return "DNSRecordSet(" + ", ".join(data) + ")"

    def __repr__(self) -> str:
        return self.__str__()


def format_record_set_for_output(
    record_set: IDNSRecordSet[RecordSetIDT_co, RecordIDT_co],
    record_name: str | None,
    prefix: str | None = None,
    record_converter: RecordConverter | None = None,
) -> dict[str, t.Any]:
    entry: dict[str, t.Any] = {
        "prefix": prefix or "",
        "type": record_set.type,
        "ttl": record_set.ttl,
        "value": sorted(record.target for record in record_set.records),
    }
    if record_converter:
        entry["value"] = record_converter.process_values_to_user(
            entry["type"], entry["value"]
        )
    if record_name is not None:
        entry["record"] = record_name
    return entry



#!/usr/bin/python
#
# Copyright (c) 2022 xuzhang3 (@xuzhang3), Fred-sun (@Fred-sun)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_privateendpointconnection

version_added: "1.12.0"

short_description: Managed private endpoint connection

description:
    - Update or delete the private endpoint connection.

options:
    name:
        description:
            - The name of the private end point connection.
        type: str
        required: True
    service_name:
        description:
            - The name of the private link service.
        type: str
        required: true
    resource_group:
        description:
            - The name of the resource group.
        type: str
        required: true
    connection_state:
        description:
            - A collection of information about the state of the connection between service consumer and provider.
        type: dict
        suboptions:
            status:
                description:
                    - Indicates whether the connection has been Approved/Rejected/Removed by the owner of the service.
                type: str
                choices:
                    - Approved
                    - Rejected
                    - Removed
            description:
                description:
                    - The reason for approval/rejection of the connection.
                type: str
            actions_required:
                description:
                    - A message indicating if changes on the service provider require any updates on the consumer.
                type: str
    state:
        description:
            - Assert the state of the connection. Use C(present) to update an connection and C(absent) to delete it.
        default: present
        type: str
        choices:
            - absent
            - present

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - xuzhang3 (@xuzhang3)
    - Fred-sun (@Fred-sun)
'''

EXAMPLES = '''
- name: Update private endpoint connection
  azure_rm_privateendpointconnection_info:
    name: pe-connection-name
    service_name: testserviceName
    resource_group: myRG
    connection_state:
      description: "new_description string"
      actions_required: "Message string"
      status: "Rejected"

- name: Delee private endpoint connection
  azure_rm_privateendpointconnection_info:
    name: pe-connection-name
    service_name: testserviceName
    resource_group: myRG
    state: absent
'''

RETURN = '''
state:
    description:
        - List of private endpoint connection info.
    returned: always
    type: complex
    contains:
        id:
            description:
                - Resource ID of the private endpoint connection.
            returned: always
            type: str
            sample: "/subscriptions/xxx/resourceGroups/myRG/providers/Microsoft.Network/privateLinkServices/linkservice/privateEndpointConnections/link.09"
        name:
            description:
                - Name of the private endpoint connection.
            returned: always
            type: str
            sample: testlink.09
        link_identifier:
            description:
                - The consumer link id.
            returned: always
            type: str
            sample: 536890208
        PrivateEndpoint:
            description:
                - The resource of private end point.
            type: complex
            returned: always
            contains:
                id:
                    description:
                        - The private endpoint resource ID.
                    type: str
                    returned: always
                    sample: "/subscriptions/xxx-xxx/resourceGroups/myRG/providers/Microsoft.Network/privateEndpoints/testlink02"
        private_link_service_connection_state:
            description:
                - A collection of information about the state of the connection between service consumer and provider.
            type: complex
            returned: always
            contains:
                description:
                    description:
                        - The reason for approval/rejection of the connection.
                    returned: always
                    type: str
                    sample: "Auto Approved"
                status:
                    description:
                        - Indicates whether the connection has been Approved/Rejected/Removed by the owner of the service.
                    returned: always
                    type: str
                    sample: Approved
                actions_required:
                    description:
                        - A message indicating if changes on the service provider require any updates on the consumer.
                    type: str
                    returned: always
                    sample: "This is action_required string"
        provisioning_state:
            description:
                - Provisioning state of the resource.
            returned: always
            type: str
            sample: Succeeded
        etag:
            description:
                - A unique read-only string that changes whenever the resource is updated.
            type: str
            returned: always
            sample: "f7d60f37-ea2b-4091-8546-1327f35468c4"
        type:
            description:
                - The resource type.
            type: str
            returned: always
            sample: Microsoft.Network/privateLinkServices/privateEndpointConnections
'''

try:
    from azure.core.exceptions import ResourceNotFoundError
    from azure.core.polling import LROPoller
except Exception:
    # This is handled in azure_rm_common
    pass

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase


connection_state_spec = dict(
    status=dict(type='str', choices=['Approved', 'Rejected', 'Removed']),
    description=dict(type='str'),
    actions_required=dict(type='str')
)


class AzureRMPrivateEndpointConnection(AzureRMModuleBase):

    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(type="str", required=True),
            service_name=dict(type="str", required=True),
            resource_group=dict(type="str", required=True),
            connection_state=dict(type='dict', options=connection_state_spec),
            state=dict(type='str', choices=['present', 'absent'], default='present'),
        )

        self.name = None
        self.service_name = None
        self.resource_group = None
        self.connection_state = None
        self.results = dict(
            changed=False,
        )

        super(AzureRMPrivateEndpointConnection, self).__init__(self.module_arg_spec,
                                                               supports_check_mode=True,
                                                               supports_tags=False,
                                                               facts_module=False)

    def exec_module(self, **kwargs):
        for key in list(self.module_arg_spec.keys()):
            setattr(self, key, kwargs[key])

        old_response = self.get_resource()
        response = None
        changed = False

        if self.state == 'present':
            if old_response:
                if self.connection_state is not None:

                    if self.connection_state.get('status') is not None:
                        if self.connection_state.get('status') != old_response['private_link_service_connection_state']['status']:
                            changed = True
                    else:
                        self.connection_state['status'] = old_response['private_link_service_connection_state']['status']

                    if self.connection_state.get('description'):
                        if self.connection_state.get('description') != old_response['private_link_service_connection_state']['description']:
                            changed = True
                    else:
                        self.connection_state['description'] = old_response['private_link_service_connection_state']['description']
                    if self.connection_state.get('actions_required'):
                        if self.connection_state.get('actions_required') != old_response['private_link_service_connection_state']['actions_required']:
                            changed = True
                    else:
                        self.connection_state['actions_required'] = old_response['private_link_service_connection_state']['actions_required']

                if changed:
                    if self.check_mode:
                        self.log("The private endpoint connection is exist, will be updated")
                    else:
                        parameters = {'private_link_service_connection_state': self.connection_state}
                        response = self.update_resource(parameters)
                        if response:
                            response = self.connect_to_dict(response)
                else:
                    if self.check_mode:
                        self.log("Check mode test. The private endpoint connection is exist, No operation in this task")
                    else:
                        response = old_response
                        self.log("The private endpoint connection is exist, No operation in this task")
            else:
                if self.check_mode:
                    changed = True
                    self.log("The private endpoint conneection is not exist, will be created, but this module not support create funciont")
                else:
                    self.fail("The private endpoint connection {0} isn't exist, This Module not support create".format(self.name))
        else:
            if old_response:
                changed = True
                if self.check_mode:
                    self.log("The private endpoint conneection is exist, will be deleted")
                else:
                    self.delete_resource()
            else:
                if self.check_mode:
                    self.log("The private endpoint connection isn't exist, no action")
                else:
                    self.log("The private endpoint connection isn't exist, don't need to delete")

        self.results['changed'] = changed
        self.results['state'] = response
        return self.results

    def get_resource(self):
        self.log("Get properties for {0} in {1}".format(self.name, self.service_name))
        try:
            response = self.network_client.private_link_services.get_private_endpoint_connection(self.resource_group, self.service_name, self.name)
            return self.connect_to_dict(response)
        except ResourceNotFoundError:
            self.log("Could not get info for {0} in {1}".format(self.name, self.service_name))

        return []

    def update_resource(self, parameters):
        self.log("Update the private endpoint connection for {0} in {1}".format(self.name, self.service_name))
        try:
            response = self.network_client.private_link_services.update_private_endpoint_connection(self.resource_group,
                                                                                                    self.service_name,
                                                                                                    self.name, parameters)
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)
            return response
        except Exception:
            self.log("Update {0} in {1} failed".format(self.name, self.service_name))

        return []

    def delete_resource(self):
        self.log("delete the private endpoint connection for {0} in {1}".format(self.name, self.service_name))
        try:
            response = self.network_client.private_link_services.begin_delete_private_endpoint_connection(self.resource_group, self.service_name, self.name)
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)
            return response
        except Exception:
            self.log("Delete {0} in {1} failed".format(self.name, self.service_name))

        return []

    def connect_to_dict(self, connect_info):
        connect = connect_info.as_dict()
        result = dict(
            id=connect.get("id"),
            name=connect.get('name'),
            type=connect.get('type'),
            etag=connect.get('etag'),
            private_endpoint=dict(),
            private_link_service_connection_state=dict(),
            provisioning_state=connect.get('provisioning_state'),
            link_identifier=connect.get('link_identifier')
        )
        if connect.get('private_endpoint') is not None:
            result['private_endpoint']['id'] = connect.get('private_endpoint')['id']

        if connect.get('private_link_service_connection_state') is not None:
            result['private_link_service_connection_state']['status'] = connect.get('private_link_service_connection_state')['status']
            result['private_link_service_connection_state']['description'] = connect.get('private_link_service_connection_state')['description']
            result['private_link_service_connection_state']['actions_required'] = connect.get('private_link_service_connection_state')['actions_required']
        return result


def main():
    AzureRMPrivateEndpointConnection()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2024, Akini Ross (@akinross) <akinross@cisco.com>

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: ndo_vlan_pool
short_description: Manage VLAN Pools on Cisco Nexus Dashboard Orchestrator (NDO).
description:
- Manage VLAN Pools on Cisco Nexus Dashboard Orchestrator (NDO).
- This module is only supported on ND v3.1 (NDO v4.3) and later.
author:
- Akini Ross (@akinross)
options:
  template:
    description:
    - The name of the template.
    - The template must be a fabric policy template.
    type: str
    required: true
  vlan_pool:
    description:
    - The name of the VLAN Pool.
    type: str
    aliases: [ name ]
  vlan_pool_uuid:
    description:
    - The uuid of the VLAN Pool.
    - This parameter is required when the O(vlan_pool) needs to be updated.
    type: str
    aliases: [ uuid ]
  description:
    description:
    - The description of the VLAN Pool.
    type: str
  vlan_ranges:
    description:
    - A list of vlan ranges attached to the VLAN Pool.
    - The list of configured vlan ranges must contain at least one entry.
    - When the list of vlan ranges is null the update will not change existing entry configuration.
    type: list
    elements: dict
    suboptions:
      from_vlan:
        description:
        - The start of the VLAN range.
        type: int
        required: true
        aliases: [ from ]
      to_vlan:
        description:
        - The end of the VLAN range.
        type: int
        required: true
        aliases: [ to ]
  state:
    description:
    - Use C(absent) for removing.
    - Use C(query) for listing an object or multiple objects.
    - Use C(present) for creating or updating.
    type: str
    choices: [ absent, query, present ]
    default: query
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Create a new vlan pool
  cisco.mso.ndo_vlan_pool:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    vlan_pool: ansible_test_vlan_pool
    vlan_ranges:
      - from_vlan: 100
        to_vlan: 200
      - from_vlan: 300
        to_vlan: 400
    state: present
  register: create

- name: Query a vlan pool with name
  cisco.mso.ndo_vlan_pool:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    vlan_pool: ansible_test_vlan_pool
    state: query
  register: query_one

- name: Query a vlan pool with UUID
  cisco.mso.ndo_vlan_pool:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    vlan_pool_uuid: '{{ create.current.uuid }}'
    state: query
  register: query_with_uuid

- name: Query all vlan pools in the template
  cisco.mso.ndo_vlan_pool:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    state: query
  register: query_all

- name: Delete a vlan pool
  cisco.mso.ndo_vlan_pool:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    vlan_pool: ansible_test_vlan_pool
    state: absent
"""

RETURN = r"""
"""


import copy
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec
from ansible_collections.cisco.mso.plugins.module_utils.template import MSOTemplate, KVPair


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        template=dict(type="str", required=True),
        vlan_pool=dict(type="str", aliases=["name"]),
        vlan_pool_uuid=dict(type="str", aliases=["uuid"]),
        description=dict(type="str"),
        vlan_ranges=dict(
            type="list",
            elements="dict",
            options=dict(
                from_vlan=dict(type="int", required=True, aliases=["from"]),
                to_vlan=dict(type="int", required=True, aliases=["to"]),
            ),
        ),
        state=dict(type="str", default="query", choices=["absent", "query", "present"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["vlan_pool"]],
            ["state", "present", ["vlan_pool"]],
        ],
    )

    mso = MSOModule(module)

    template = module.params.get("template")
    vlan_pool = module.params.get("vlan_pool")
    vlan_pool_uuid = module.params.get("vlan_pool_uuid")
    vlan_ranges = get_vlan_ranges_payload(module.params.get("vlan_ranges")) if module.params.get("vlan_ranges") else []
    description = module.params.get("description")
    state = module.params.get("state")

    ops = []
    match = None

    mso_template = MSOTemplate(mso, "fabric_policy", template)
    mso_template.validate_template("fabricPolicy")

    path = "/fabricPolicyTemplate/template/vlanPools"
    match = get_fabric_policy_vlan_pool(mso_template, vlan_pool_uuid, vlan_pool)

    if vlan_pool_uuid or vlan_pool:
        if match:
            match.details["vlan_ranges"] = match.details["encapBlocks"]
            mso.existing = mso.previous = copy.deepcopy(match.details)  # Query a specific object
    elif match:
        mso.existing = [vlan_pool.update({"vlan_ranges": vlan_pool["encapBlocks"]}) for vlan_pool in match]  # Query all objects

    if state == "present":
        err_message_min_vlan_ranges = "At least one vlan range is required when state is present."

        if match:
            if module.params.get("vlan_ranges") is not None and len(vlan_ranges) == 0:
                mso.fail_json(msg=err_message_min_vlan_ranges)

            if vlan_pool and match.details.get("name") != vlan_pool:
                ops.append(dict(op="replace", path="{0}/{1}/name".format(path, match.index), value=vlan_pool))
                match.details["name"] = vlan_pool

            if description is not None and match.details.get("description") != description:
                ops.append(dict(op="replace", path="{0}/{1}/description".format(path, match.index), value=description))
                match.details["description"] = description

            if module.params.get("vlan_ranges") is not None and match.details.get("encapBlocks") != vlan_ranges:
                ops.append(dict(op="replace", path="{0}/{1}/encapBlocks".format(path, match.index), value=vlan_ranges))
                match.details["vlan_ranges"] = vlan_ranges
                match.details["encapBlocks"] = vlan_ranges

            mso.sanitize(match.details)

        else:
            if not vlan_ranges:
                mso.fail_json(msg=err_message_min_vlan_ranges)

            payload = {"name": vlan_pool, "allocMode": "static", "encapBlocks": vlan_ranges}
            if description:
                payload["description"] = description

            ops.append(dict(op="add", path="{0}/-".format(path), value=copy.deepcopy(payload)))

            payload["vlan_ranges"] = vlan_ranges
            mso.sanitize(payload)
        mso.existing = mso.proposed

    elif state == "absent":
        if match:
            ops.append(dict(op="remove", path="{0}/{1}".format(path, match.index)))
        mso.existing = {}

    if not module.check_mode and ops:
        mso_template.template = mso.request(mso_template.template_path, method="PATCH", data=ops)
        match = get_fabric_policy_vlan_pool(mso_template, vlan_pool_uuid, vlan_pool)
        if match:
            match.details["vlan_ranges"] = match.details["encapBlocks"]
            mso.existing = match.details  # When the state is present
        else:
            mso.existing = {}  # When the state is absent
    elif module.check_mode and state != "query":  # When the state is present/absent with check mode
        mso.existing = mso.proposed if state == "present" else {}

    mso.exit_json()


def get_vlan_ranges_payload(vlan_ranges):
    payload = []
    for vlan_range in vlan_ranges:
        vlan_range_payload = {
            "range": {
                "from": vlan_range.get("from_vlan"),
                "to": vlan_range.get("to_vlan"),
                "allocMode": "static",
            }
        }
        payload.append(vlan_range_payload)
    return payload


def get_fabric_policy_vlan_pool(mso_template, uuid=None, name=None, fail_module=False):
    """
    Get the Fabric Policy VLAN Pool by UUID or Name.
    :param uuid: UUID of the Fabric Policy VLAN Pool to search for -> Str
    :param name: Name of the Fabric Policy VLAN Pool to search for -> Str
    :param fail_module: When match is not found fail the ansible module -> Bool
    :return: Dict | None | List[Dict] | List[]: The processed result which could be:
              When the UUID | Name is existing in the search list -> Dict
              When the UUID | Name is not existing in the search list -> None
              When both UUID and Name are None, and the search list is not empty -> List[Dict]
              When both UUID and Name are None, and the search list is empty -> List[]
    """
    match = mso_template.template.get("fabricPolicyTemplate", {}).get("template", {}).get("vlanPools", [])
    if uuid or name:  # Query a specific object
        return mso_template.get_object_by_key_value_pairs("VLAN Pool", match, [KVPair("uuid", uuid) if uuid else KVPair("name", name)], fail_module)
    return match  # Query all objects


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2014, Richard Isaacson <richard.c.isaacson@gmail.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = r'''
---
module: at
short_description: Schedule the execution of a command or script file via the at command
description:
 - Use this module to schedule a command or script file to run once in the future.
 - All jobs are executed in the 'a' queue.
version_added: "1.0.0"
options:
  command:
    description:
     - A command to be executed in the future.
    type: str
  script_file:
    description:
     - An existing script file to be executed in the future.
    type: str
  count:
    description:
     - The count of units in the future to execute the command or script file.
    type: int
  units:
    description:
     - The type of units in the future to execute the command or script file.
    type: str
    choices: [ minutes, hours, days, weeks ]
  state:
    description:
     - The state dictates if the command or script file should be evaluated as V(present) (added) or V(absent) (deleted).
    type: str
    choices: [ absent, present ]
    default: present
  unique:
    description:
     - If a matching job is present a new job will not be added.
    type: bool
    default: false
requirements:
 - at
author:
- Richard Isaacson (@risaacson)
'''

EXAMPLES = r'''
- name: Schedule a command to execute in 20 minutes as root
  ansible.posix.at:
    command: ls -d / >/dev/null
    count: 20
    units: minutes

- name: Match a command to an existing job and delete the job
  ansible.posix.at:
    command: ls -d / >/dev/null
    state: absent

- name: Schedule a command to execute in 20 minutes making sure it is unique in the queue
  ansible.posix.at:
    command: ls -d / >/dev/null
    count: 20
    units: minutes
    unique: true
'''

import os
import platform
import tempfile

from ansible.module_utils.basic import AnsibleModule


def add_job(module, result, at_cmd, count, units, command, script_file):
    at_command = "%s -f %s now + %s %s" % (at_cmd, script_file, count, units)
    rc, out, err = module.run_command(at_command, check_rc=True)
    if command:
        os.unlink(script_file)
    result['changed'] = True


def delete_job(module, result, at_cmd, command, script_file):
    for matching_job in get_matching_jobs(module, at_cmd, script_file):
        at_command = "%s -r %s" % (at_cmd, matching_job)
        rc, out, err = module.run_command(at_command, check_rc=True)
        result['changed'] = True
    if command:
        os.unlink(script_file)
    module.exit_json(**result)


def get_matching_jobs(module, at_cmd, script_file):
    matching_jobs = []

    atq_cmd = module.get_bin_path('atq', True)

    # Get list of job numbers for the user.
    atq_command = "%s" % atq_cmd
    rc, out, err = module.run_command(atq_command, check_rc=True)
    current_jobs = out.splitlines()
    if len(current_jobs) == 0:
        return matching_jobs

    # Read script_file into a string.
    with open(script_file) as script_fh:
        script_file_string = script_fh.read().strip()

    # Loop through the jobs.
    #   If the script text is contained in a job add job number to list.
    for current_job in current_jobs:
        split_current_job = current_job.split()
        at_opt = '-c' if platform.system() != 'AIX' else '-lv'
        at_command = "%s %s %s" % (at_cmd, at_opt, split_current_job[0])
        rc, out, err = module.run_command(at_command, check_rc=True)
        if script_file_string in out:
            matching_jobs.append(split_current_job[0])

    # Return the list.
    return matching_jobs


def create_tempfile(command):
    filed, script_file = tempfile.mkstemp(prefix='at')
    fileh = os.fdopen(filed, 'w')
    fileh.write(command + os.linesep)
    fileh.close()
    return script_file


def main():

    module = AnsibleModule(
        argument_spec=dict(
            command=dict(type='str'),
            script_file=dict(type='str'),
            count=dict(type='int'),
            units=dict(type='str', choices=['minutes', 'hours', 'days', 'weeks']),
            state=dict(type='str', default='present', choices=['absent', 'present']),
            unique=dict(type='bool', default=False),
        ),
        mutually_exclusive=[['command', 'script_file']],
        required_one_of=[['command', 'script_file']],
        supports_check_mode=False,
    )

    at_cmd = module.get_bin_path('at', True)

    command = module.params['command']
    script_file = module.params['script_file']
    count = module.params['count']
    units = module.params['units']
    state = module.params['state']
    unique = module.params['unique']

    if (state == 'present') and (not count or not units):
        module.fail_json(msg="present state requires count and units")

    result = dict(
        changed=False,
        state=state,
    )

    # If command transform it into a script_file
    if command:
        script_file = create_tempfile(command)

    # if absent remove existing and return
    if state == 'absent':
        delete_job(module, result, at_cmd, command, script_file)

    # if unique if existing return unchanged
    if unique:
        if len(get_matching_jobs(module, at_cmd, script_file)) != 0:
            if command:
                os.unlink(script_file)
            module.exit_json(**result)

    result['script_file'] = script_file
    result['count'] = count
    result['units'] = units

    add_job(module, result, at_cmd, count, units, command, script_file)

    module.exit_json(**result)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# Copyright 2024 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

#############################################
#                WARNING                    #
#############################################
#
# This file is auto generated by the
# ansible.content_builder.
#
# Manually editing this file is not advised.
#
# To update the argspec make the desired changes
# in the documentation in the module file and re-run
# ansible.content_builder commenting out
# the path to external 'docstring' in build.yaml.
#
##############################################

"""
The arg spec for the ios_vrf_address_family module
"""


class Vrf_address_familyArgs(object):  # pylint: disable=R0903
    """The arg spec for the ios_vrf_address_family module"""

    argument_spec = {
        "config": {
            "type": "list",
            "elements": "dict",
            "options": {
                "name": {"type": "str", "required": True},
                "address_families": {
                    "type": "list",
                    "elements": "dict",
                    "options": {
                        "afi": {"type": "str", "choices": ["ipv4", "ipv6"]},
                        "safi": {
                            "type": "str",
                            "choices": ["multicast", "unicast"],
                        },
                        "bgp": {
                            "type": "dict",
                            "options": {
                                "next_hop": {
                                    "type": "dict",
                                    "options": {"loopback": {"type": "int"}},
                                },
                            },
                        },
                        "export": {
                            "type": "dict",
                            "options": {
                                "map": {"type": "str"},
                                "ipv4": {
                                    "type": "dict",
                                    "options": {
                                        "multicast": {
                                            "type": "dict",
                                            "options": {
                                                "prefix": {"type": "int"},
                                                "map": {"type": "str"},
                                            },
                                        },
                                        "unicast": {
                                            "type": "dict",
                                            "options": {
                                                "prefix": {"type": "int"},
                                                "map": {"type": "str"},
                                                "allow_evpn": {"type": "bool"},
                                            },
                                        },
                                    },
                                },
                            },
                        },
                        "import_config": {
                            "type": "dict",
                            "options": {
                                "map": {"type": "str"},
                                "ipv4": {
                                    "type": "dict",
                                    "options": {
                                        "multicast": {
                                            "type": "dict",
                                            "options": {
                                                "prefix": {"type": "int"},
                                                "map": {"type": "str"},
                                            },
                                        },
                                        "unicast": {
                                            "type": "dict",
                                            "options": {
                                                "limit": {"type": "int"},
                                                "map": {"type": "str"},
                                                "allow_evpn": {"type": "bool"},
                                            },
                                        },
                                    },
                                },
                            },
                        },
                        "inter_as_hybrid": {
                            "type": "dict",
                            "options": {
                                "csc": {
                                    "type": "dict",
                                    "options": {"next_hop": {"type": "str"}},
                                },
                                "next_hop": {"type": "str"},
                            },
                        },
                        "mdt": {
                            "type": "dict",
                            "options": {
                                "auto_discovery": {
                                    "type": "dict",
                                    "options": {
                                        "ingress_replication": {
                                            "type": "dict",
                                            "options": {
                                                "inter_as": {
                                                    "type": "dict",
                                                    "options": {
                                                        "mdt_hello_enable": {
                                                            "type": "bool",
                                                        },
                                                    },
                                                },
                                                "mdt_hello_enable": {
                                                    "type": "bool",
                                                },
                                            },
                                        },
                                        "mldp": {"type": "bool"},
                                        "pim": {
                                            "type": "dict",
                                            "options": {
                                                "inter_as": {
                                                    "type": "dict",
                                                    "options": {
                                                        "mdt_hello_enable": {
                                                            "type": "bool",
                                                        },
                                                        "pim_tlv_announce": {
                                                            "type": "dict",
                                                            "options": {
                                                                "mdt_hello_enable": {
                                                                    "type": "bool",
                                                                },
                                                            },
                                                        },
                                                    },
                                                },
                                                "mdt_hello_enable": {
                                                    "type": "bool",
                                                },
                                                "pim_tlv_announce": {
                                                    "type": "dict",
                                                    "options": {
                                                        "mdt_hello_enable": {
                                                            "type": "bool",
                                                        },
                                                    },
                                                },
                                            },
                                        },
                                        "receiver_site": {"type": "bool"},
                                    },
                                },
                                "data": {
                                    "type": "dict",
                                    "options": {
                                        "ingress_replication": {
                                            "type": "dict",
                                            "options": {
                                                "number": {"type": "int"},
                                                "immediate_switch": {
                                                    "type": "bool",
                                                },
                                                "list": {
                                                    "type": "dict",
                                                    "options": {
                                                        "access_list_number": {
                                                            "type": "int",
                                                        },
                                                        "access_list_name": {
                                                            "type": "str",
                                                        },
                                                    },
                                                },
                                            },
                                        },
                                        "list": {
                                            "type": "dict",
                                            "options": {
                                                "access_list_number": {
                                                    "type": "int",
                                                },
                                                "access_list_name": {
                                                    "type": "str",
                                                },
                                            },
                                        },
                                        "threshold": {"type": "int"},
                                        "mpls": {
                                            "type": "dict",
                                            "options": {
                                                "mldp": {"type": "int"},
                                            },
                                        },
                                    },
                                },
                                "default": {
                                    "type": "dict",
                                    "options": {
                                        "ingress_replication": {"type": "bool"},
                                    },
                                },
                                "direct": {"type": "bool"},
                                "log_reuse": {"type": "bool"},
                                "mode": {
                                    "type": "dict",
                                    "options": {"gre": {"type": "bool"}},
                                },
                                "mtu": {"type": "int"},
                                "overlay": {
                                    "type": "dict",
                                    "options": {
                                        "bgp": {
                                            "type": "dict",
                                            "options": {
                                                "shared_tree_prune_delay": {
                                                    "type": "int",
                                                },
                                                "source_tree_prune_delay": {
                                                    "type": "int",
                                                },
                                            },
                                        },
                                        "use_bgp": {
                                            "type": "dict",
                                            "options": {
                                                "set": {"type": "bool"},
                                                "spt_only": {"type": "bool"},
                                            },
                                        },
                                    },
                                },
                                "partitioned": {
                                    "type": "dict",
                                    "options": {
                                        "ingress_replication": {"type": "bool"},
                                        "mldp": {
                                            "type": "dict",
                                            "options": {
                                                "p2mp": {"type": "bool"},
                                            },
                                        },
                                    },
                                },
                                "strict_rpf": {
                                    "type": "dict",
                                    "options": {"interface": {"type": "bool"}},
                                },
                            },
                        },
                        "protection": {
                            "type": "dict",
                            "options": {"local_prefixes": {"type": "bool"}},
                        },
                        "route_replicate": {
                            "type": "dict",
                            "options": {
                                "recursion_policy": {
                                    "type": "dict",
                                    "options": {"destination": {"type": "bool"}},
                                },
                                "from_config": {
                                    "type": "dict",
                                    "options": {
                                        "multicast": {
                                            "type": "dict",
                                            "options": {
                                                "all": {
                                                    "type": "dict",
                                                    "options": {
                                                        "route_map": {
                                                            "type": "str",
                                                        },
                                                    },
                                                },
                                                "bgp": {
                                                    "type": "dict",
                                                    "options": {
                                                        "as_number": {
                                                            "type": "int",
                                                        },
                                                        "route_map": {
                                                            "type": "str",
                                                        },
                                                    },
                                                },
                                                "eigrp": {
                                                    "type": "dict",
                                                    "options": {
                                                        "as_number": {
                                                            "type": "int",
                                                        },
                                                        "route_map": {
                                                            "type": "str",
                                                        },
                                                    },
                                                },
                                                "isis": {
                                                    "type": "dict",
                                                    "options": {
                                                        "iso_tag": {"type": "str"},
                                                        "route_map": {
                                                            "type": "str",
                                                        },
                                                    },
                                                },
                                                "mobile": {
                                                    "type": "dict",
                                                    "options": {
                                                        "route_map": {
                                                            "type": "str",
                                                        },
                                                    },
                                                },
                                                "odr": {
                                                    "type": "dict",
                                                    "options": {
                                                        "route_map": {
                                                            "type": "str",
                                                        },
                                                    },
                                                },
                                                "ospf": {
                                                    "type": "dict",
                                                    "options": {
                                                        "process_id": {
                                                            "type": "int",
                                                        },
                                                        "route_map": {
                                                            "type": "str",
                                                        },
                                                    },
                                                },
                                                "rip": {
                                                    "type": "dict",
                                                    "options": {
                                                        "route_map": {
                                                            "type": "str",
                                                        },
                                                    },
                                                },
                                                "static": {
                                                    "type": "dict",
                                                    "options": {
                                                        "route_map": {
                                                            "type": "str",
                                                        },
                                                    },
                                                },
                                                "topology": {
                                                    "type": "dict",
                                                    "options": {
                                                        "base": {
                                                            "type": "dict",
                                                            "options": {
                                                                "all": {
                                                                    "type": "dict",
                                                                    "options": {
                                                                        "route_map": {
                                                                            "type": "str",
                                                                        },
                                                                    },
                                                                },
                                                                "bgp": {
                                                                    "type": "dict",
                                                                    "options": {
                                                                        "as_number": {
                                                                            "type": "int",
                                                                        },
                                                                        "route_map": {
                                                                            "type": "str",
                                                                        },
                                                                    },
                                                                },
                                                                "eigrp": {
                                                                    "type": "dict",
                                                                    "options": {
                                                                        "as_number": {
                                                                            "type": "int",
                                                                        },
                                                                        "route_map": {
                                                                            "type": "str",
                                                                        },
                                                                    },
                                                                },
                                                                "isis": {
                                                                    "type": "dict",
                                                                    "options": {
                                                                        "iso_tag": {
                                                                            "type": "str",
                                                                        },
                                                                        "route_map": {
                                                                            "type": "str",
                                                                        },
                                                                    },
                                                                },
                                                                "mobile": {
                                                                    "type": "dict",
                                                                    "options": {
                                                                        "route_map": {
                                                                            "type": "str",
                                                                        },
                                                                    },
                                                                },
                                                                "odr": {
                                                                    "type": "dict",
                                                                    "options": {
                                                                        "route_map": {
                                                                            "type": "str",
                                                                        },
                                                                    },
                                                                },
                                                                "ospf": {
                                                                    "type": "dict",
                                                                    "options": {
                                                                        "process_id": {
                                                                            "type": "int",
                                                                        },
                                                                        "route_map": {
                                                                            "type": "str",
                                                                        },
                                                                    },
                                                                },
                                                                "rip": {
                                                                    "type": "dict",
                                                                    "options": {
                                                                        "route_map": {
                                                                            "type": "str",
                                                                        },
                                                                    },
                                                                },
                                                                "static": {
                                                                    "type": "dict",
                                                                    "options": {
                                                                        "route_map": {
                                                                            "type": "str",
                                                                        },
                                                                    },
                                                                },
                                                            },
                                                        },
                                                    },
                                                },
                                            },
                                        },
                                        "unicast": {
                                            "type": "dict",
                                            "options": {
                                                "all": {
                                                    "type": "dict",
                                                    "options": {
                                                        "route_map": {
                                                            "type": "str",
                                                        },
                                                    },
                                                },
                                                "bgp": {
                                                    "type": "dict",
                                                    "options": {
                                                        "as_number": {
                                                            "type": "int",
                                                        },
                                                        "route_map": {
                                                            "type": "str",
                                                        },
                                                    },
                                                },
                                                "connected": {
                                                    "type": "dict",
                                                    "options": {
                                                        "route_map": {
                                                            "type": "str",
                                                        },
                                                    },
                                                },
                                                "eigrp": {
                                                    "type": "dict",
                                                    "options": {
                                                        "as_number": {
                                                            "type": "int",
                                                        },
                                                        "route_map": {
                                                            "type": "str",
                                                        },
                                                    },
                                                },
                                                "isis": {
                                                    "type": "dict",
                                                    "options": {
                                                        "iso_tag": {"type": "str"},
                                                        "route_map": {
                                                            "type": "str",
                                                        },
                                                    },
                                                },
                                                "mobile": {
                                                    "type": "dict",
                                                    "options": {
                                                        "route_map": {
                                                            "type": "str",
                                                        },
                                                    },
                                                },
                                                "odr": {
                                                    "type": "dict",
                                                    "options": {
                                                        "route_map": {
                                                            "type": "str",
                                                        },
                                                    },
                                                },
                                                "ospf": {
                                                    "type": "dict",
                                                    "options": {
                                                        "process_id": {
                                                            "type": "int",
                                                        },
                                                        "route_map": {
                                                            "type": "str",
                                                        },
                                                    },
                                                },
                                                "rip": {
                                                    "type": "dict",
                                                    "options": {
                                                        "route_map": {
                                                            "type": "str",
                                                        },
                                                    },
                                                },
                                                "static": {
                                                    "type": "dict",
                                                    "options": {
                                                        "route_map": {
                                                            "type": "str",
                                                        },
                                                    },
                                                },
                                            },
                                        },
                                        "vrf": {
                                            "type": "dict",
                                            "options": {
                                                "name": {"type": "str"},
                                                "multicast": {
                                                    "type": "dict",
                                                    "options": {
                                                        "all": {
                                                            "type": "dict",
                                                            "options": {
                                                                "route_map": {
                                                                    "type": "str",
                                                                },
                                                            },
                                                        },
                                                        "bgp": {
                                                            "type": "dict",
                                                            "options": {
                                                                "as_number": {
                                                                    "type": "int",
                                                                },
                                                                "route_map": {
                                                                    "type": "str",
                                                                },
                                                            },
                                                        },
                                                        "eigrp": {
                                                            "type": "dict",
                                                            "options": {
                                                                "as_number": {
                                                                    "type": "int",
                                                                },
                                                                "route_map": {
                                                                    "type": "str",
                                                                },
                                                            },
                                                        },
                                                        "isis": {
                                                            "type": "dict",
                                                            "options": {
                                                                "iso_tag": {
                                                                    "type": "str",
                                                                },
                                                                "route_map": {
                                                                    "type": "str",
                                                                },
                                                            },
                                                        },
                                                        "mobile": {
                                                            "type": "dict",
                                                            "options": {
                                                                "route_map": {
                                                                    "type": "str",
                                                                },
                                                            },
                                                        },
                                                        "odr": {
                                                            "type": "dict",
                                                            "options": {
                                                                "route_map": {
                                                                    "type": "str",
                                                                },
                                                            },
                                                        },
                                                        "ospf": {
                                                            "type": "dict",
                                                            "options": {
                                                                "process_id": {
                                                                    "type": "int",
                                                                },
                                                                "route_map": {
                                                                    "type": "str",
                                                                },
                                                            },
                                                        },
                                                        "rip": {
                                                            "type": "dict",
                                                            "options": {
                                                                "route_map": {
                                                                    "type": "str",
                                                                },
                                                            },
                                                        },
                                                        "static": {
                                                            "type": "dict",
                                                            "options": {
                                                                "route_map": {
                                                                    "type": "str",
                                                                },
                                                            },
                                                        },
                                                        "topology": {
                                                            "type": "dict",
                                                            "options": {
                                                                "base": {
                                                                    "type": "dict",
                                                                    "options": {
                                                                        "all": {
                                                                            "type": "dict",
                                                                            "options": {
                                                                                "route_map": {
                                                                                    "type": "str",
                                                                                },
                                                                            },
                                                                        },
                                                                        "bgp": {
                                                                            "type": "dict",
                                                                            "options": {
                                                                                "as_number": {
                                                                                    "type": "int",
                                                                                },
                                                                                "route_map": {
                                                                                    "type": "str",
                                                                                },
                                                                            },
                                                                        },
                                                                        "eigrp": {
                                                                            "type": "dict",
                                                                            "options": {
                                                                                "as_number": {
                                                                                    "type": "int",
                                                                                },
                                                                                "route_map": {
                                                                                    "type": "str",
                                                                                },
                                                                            },
                                                                        },
                                                                        "isis": {
                                                                            "type": "dict",
                                                                            "options": {
                                                                                "iso_tag": {
                                                                                    "type": "str",
                                                                                },
                                                                                "route_map": {
                                                                                    "type": "str",
                                                                                },
                                                                            },
                                                                        },
                                                                        "mobile": {
                                                                            "type": "dict",
                                                                            "options": {
                                                                                "route_map": {
                                                                                    "type": "str",
                                                                                },
                                                                            },
                                                                        },
                                                                        "odr": {
                                                                            "type": "dict",
                                                                            "options": {
                                                                                "route_map": {
                                                                                    "type": "str",
                                                                                },
                                                                            },
                                                                        },
                                                                        "ospf": {
                                                                            "type": "dict",
                                                                            "options": {
                                                                                "process_id": {
                                                                                    "type": "int",
                                                                                },
                                                                                "route_map": {
                                                                                    "type": "str",
                                                                                },
                                                                            },
                                                                        },
                                                                        "rip": {
                                                                            "type": "dict",
                                                                            "options": {
                                                                                "route_map": {
                                                                                    "type": "str",
                                                                                },
                                                                            },
                                                                        },
                                                                        "static": {
                                                                            "type": "dict",
                                                                            "options": {
                                                                                "route_map": {
                                                                                    "type": "str",
                                                                                },
                                                                            },
                                                                        },
                                                                    },
                                                                },
                                                            },
                                                        },
                                                    },
                                                },
                                                "unicast": {
                                                    "type": "dict",
                                                    "options": {
                                                        "all": {
                                                            "type": "dict",
                                                            "options": {
                                                                "route_map": {
                                                                    "type": "str",
                                                                },
                                                            },
                                                        },
                                                        "bgp": {
                                                            "type": "dict",
                                                            "options": {
                                                                "as_number": {
                                                                    "type": "int",
                                                                },
                                                                "route_map": {
                                                                    "type": "str",
                                                                },
                                                            },
                                                        },
                                                        "connected": {
                                                            "type": "dict",
                                                            "options": {
                                                                "route_map": {
                                                                    "type": "str",
                                                                },
                                                            },
                                                        },
                                                        "eigrp": {
                                                            "type": "dict",
                                                            "options": {
                                                                "as_number": {
                                                                    "type": "int",
                                                                },
                                                                "route_map": {
                                                                    "type": "str",
                                                                },
                                                            },
                                                        },
                                                        "isis": {
                                                            "type": "dict",
                                                            "options": {
                                                                "iso_tag": {
                                                                    "type": "str",
                                                                },
                                                                "route_map": {
                                                                    "type": "str",
                                                                },
                                                            },
                                                        },
                                                        "mobile": {
                                                            "type": "dict",
                                                            "options": {
                                                                "route_map": {
                                                                    "type": "str",
                                                                },
                                                            },
                                                        },
                                                        "odr": {
                                                            "type": "dict",
                                                            "options": {
                                                                "route_map": {
                                                                    "type": "str",
                                                                },
                                                            },
                                                        },
                                                        "ospf": {
                                                            "type": "dict",
                                                            "options": {
                                                                "process_id": {
                                                                    "type": "int",
                                                                },
                                                                "route_map": {
                                                                    "type": "str",
                                                                },
                                                            },
                                                        },
                                                        "rip": {
                                                            "type": "dict",
                                                            "options": {
                                                                "route_map": {
                                                                    "type": "str",
                                                                },
                                                            },
                                                        },
                                                        "static": {
                                                            "type": "dict",
                                                            "options": {
                                                                "route_map": {
                                                                    "type": "str",
                                                                },
                                                            },
                                                        },
                                                    },
                                                },
                                                "global": {
                                                    "type": "dict",
                                                    "options": {
                                                        "multicast": {
                                                            "type": "dict",
                                                            "options": {
                                                                "all": {
                                                                    "type": "dict",
                                                                    "options": {
                                                                        "route_map": {
                                                                            "type": "str",
                                                                        },
                                                                    },
                                                                },
                                                                "bgp": {
                                                                    "type": "dict",
                                                                    "options": {
                                                                        "as_number": {
                                                                            "type": "int",
                                                                        },
                                                                        "route_map": {
                                                                            "type": "str",
                                                                        },
                                                                    },
                                                                },
                                                                "eigrp": {
                                                                    "type": "dict",
                                                                    "options": {
                                                                        "as_number": {
                                                                            "type": "int",
                                                                        },
                                                                        "route_map": {
                                                                            "type": "str",
                                                                        },
                                                                    },
                                                                },
                                                                "isis": {
                                                                    "type": "dict",
                                                                    "options": {
                                                                        "iso_tag": {
                                                                            "type": "str",
                                                                        },
                                                                        "route_map": {
                                                                            "type": "str",
                                                                        },
                                                                    },
                                                                },
                                                                "mobile": {
                                                                    "type": "dict",
                                                                    "options": {
                                                                        "route_map": {
                                                                            "type": "str",
                                                                        },
                                                                    },
                                                                },
                                                                "odr": {
                                                                    "type": "dict",
                                                                    "options": {
                                                                        "route_map": {
                                                                            "type": "str",
                                                                        },
                                                                    },
                                                                },
                                                                "ospf": {
                                                                    "type": "dict",
                                                                    "options": {
                                                                        "process_id": {
                                                                            "type": "int",
                                                                        },
                                                                        "route_map": {
                                                                            "type": "str",
                                                                        },
                                                                    },
                                                                },
                                                                "rip": {
                                                                    "type": "dict",
                                                                    "options": {
                                                                        "route_map": {
                                                                            "type": "str",
                                                                        },
                                                                    },
                                                                },
                                                                "static": {
                                                                    "type": "dict",
                                                                    "options": {
                                                                        "route_map": {
                                                                            "type": "str",
                                                                        },
                                                                    },
                                                                },
                                                                "topology": {
                                                                    "type": "dict",
                                                                    "options": {
                                                                        "base": {
                                                                            "type": "dict",
                                                                            "options": {
                                                                                "all": {
                                                                                    "type": "dict",
                                                                                    "options": {
                                                                                        "route_map": {
                                                                                            "type": "str",
                                                                                        },
                                                                                    },
                                                                                },
                                                                                "bgp": {
                                                                                    "type": "dict",
                                                                                    "options": {
                                                                                        "as_number": {
                                                                                            "type": "int",
                                                                                        },
                                                                                        "route_map": {
                                                                                            "type": "str",
                                                                                        },
                                                                                    },
                                                                                },
                                                                                "eigrp": {
                                                                                    "type": "dict",
                                                                                    "options": {
                                                                                        "as_number": {
                                                                                            "type": "int",
                                                                                        },
                                                                                        "route_map": {
                                                                                            "type": "str",
                                                                                        },
                                                                                    },
                                                                                },
                                                                                "isis": {
                                                                                    "type": "dict",
                                                                                    "options": {
                                                                                        "iso_tag": {
                                                                                            "type": "str",
                                                                                        },
                                                                                        "route_map": {
                                                                                            "type": "str",
                                                                                        },
                                                                                    },
                                                                                },
                                                                                "mobile": {
                                                                                    "type": "dict",
                                                                                    "options": {
                                                                                        "route_map": {
                                                                                            "type": "str",
                                                                                        },
                                                                                    },
                                                                                },
                                                                                "odr": {
                                                                                    "type": "dict",
                                                                                    "options": {
                                                                                        "route_map": {
                                                                                            "type": "str",
                                                                                        },
                                                                                    },
                                                                                },
                                                                                "ospf": {
                                                                                    "type": "dict",
                                                                                    "options": {
                                                                                        "process_id": {
                                                                                            "type": "int",
                                                                                        },
                                                                                        "route_map": {
                                                                                            "type": "str",
                                                                                        },
                                                                                    },
                                                                                },
                                                                                "rip": {
                                                                                    "type": "dict",
                                                                                    "options": {
                                                                                        "route_map": {
                                                                                            "type": "str",
                                                                                        },
                                                                                    },
                                                                                },
                                                                                "static": {
                                                                                    "type": "dict",
                                                                                    "options": {
                                                                                        "route_map": {
                                                                                            "type": "str",
                                                                                        },
                                                                                    },
                                                                                },
                                                                            },
                                                                        },
                                                                    },
                                                                },
                                                            },
                                                        },
                                                        "unicast": {
                                                            "type": "dict",
                                                            "options": {
                                                                "all": {
                                                                    "type": "dict",
                                                                    "options": {
                                                                        "route_map": {
                                                                            "type": "str",
                                                                        },
                                                                    },
                                                                },
                                                                "bgp": {
                                                                    "type": "dict",
                                                                    "options": {
                                                                        "as_number": {
                                                                            "type": "int",
                                                                        },
                                                                        "route_map": {
                                                                            "type": "str",
                                                                        },
                                                                    },
                                                                },
                                                                "connected": {
                                                                    "type": "dict",
                                                                    "options": {
                                                                        "route_map": {
                                                                            "type": "str",
                                                                        },
                                                                    },
                                                                },
                                                                "eigrp": {
                                                                    "type": "dict",
                                                                    "options": {
                                                                        "as_number": {
                                                                            "type": "int",
                                                                        },
                                                                        "route_map": {
                                                                            "type": "str",
                                                                        },
                                                                    },
                                                                },
                                                                "isis": {
                                                                    "type": "dict",
                                                                    "options": {
                                                                        "iso_tag": {
                                                                            "type": "str",
                                                                        },
                                                                        "route_map": {
                                                                            "type": "str",
                                                                        },
                                                                    },
                                                                },
                                                                "mobile": {
                                                                    "type": "dict",
                                                                    "options": {
                                                                        "route_map": {
                                                                            "type": "str",
                                                                        },
                                                                    },
                                                                },
                                                                "odr": {
                                                                    "type": "dict",
                                                                    "options": {
                                                                        "route_map": {
                                                                            "type": "str",
                                                                        },
                                                                    },
                                                                },
                                                                "ospf": {
                                                                    "type": "dict",
                                                                    "options": {
                                                                        "process_id": {
                                                                            "type": "int",
                                                                        },
                                                                        "route_map": {
                                                                            "type": "str",
                                                                        },
                                                                    },
                                                                },
                                                                "rip": {
                                                                    "type": "dict",
                                                                    "options": {
                                                                        "route_map": {
                                                                            "type": "str",
                                                                        },
                                                                    },
                                                                },
                                                                "static": {
                                                                    "type": "dict",
                                                                    "options": {
                                                                        "route_map": {
                                                                            "type": "str",
                                                                        },
                                                                    },
                                                                },
                                                            },
                                                        },
                                                    },
                                                },
                                            },
                                        },
                                    },
                                },
                            },
                        },
                        "route_target": {
                            "type": "dict",
                            "options": {
                                "export": {"type": "str"},
                                "import_config": {"type": "str"},
                                "both": {"type": "str"},
                                "exports": {
                                    "type": "list",
                                    "elements": "dict",
                                    "options": {
                                        "rt_value": {"type": "str"},
                                        "stitching": {"type": "bool"},
                                    },
                                },
                                "imports": {
                                    "type": "list",
                                    "elements": "dict",
                                    "options": {
                                        "rt_value": {"type": "str"},
                                        "stitching": {"type": "bool"},
                                    },
                                },
                                "both_options": {
                                    "type": "list",
                                    "elements": "dict",
                                    "options": {
                                        "rt_value": {"type": "str"},
                                        "stitching": {"type": "bool"},
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        "running_config": {"type": "str"},
        "state": {
            "choices": [
                "parsed",
                "gathered",
                "deleted",
                "merged",
                "replaced",
                "rendered",
                "overridden",
            ],
            "default": "merged",
            "type": "str",
        },
    }  # pylint: disable=C0301



# -*- coding: utf-8 -*-

# Copyright: (c) 2016 Matt Davis, <mdavis@ansible.com>
# Copyright: (c) 2016 Chris Houseknecht, <house@redhat.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


class ModuleDocFragment(object):

    # Azure doc fragment
    DOCUMENTATION = r'''

options:
    subscription_id:
        description:
            - Your Azure subscription Id.
        type: str
        env:
          - name: AZURE_SUBSCRIPTION_ID
    client_id:
        description:
            - Azure client ID. Use when authenticating with a Service Principal or Managed Identity (msi).
            - Can also be set via the C(AZURE_CLIENT_ID) environment variable.
        type: str
        env:
          - name: AZURE_CLIENT_ID
    secret:
        description:
            - Azure client secret. Use when authenticating with a Service Principal.
        type: str
        env:
          - name: AZURE_SECRET
    tenant:
        description:
            - Azure tenant ID. Use when authenticating with a Service Principal.
        type: str
        env:
          - name: AZURE_TENANT
    oidc_token:
        description:
            - Direct OpenID Connect (OIDC) token (JWT) used for workload identity authentication.
        type: str
        env:
          - name: AZURE_FEDERATED_TOKEN
    oidc_token_file_path:
        description:
            - Path to a file containing an OpenID Connect (OIDC) token (JWT) used for workload identity authentication.
        type: path
        env:
          - name: AZURE_FEDERATED_TOKEN_FILE
    cloud_environment:
        description:
            - For cloud environments other than the US public cloud, the environment name (as defined by Azure Python SDK, eg, C(AzureChinaCloud),
              C(AzureUSGovernment)), or a resource manager URL (required for Azure Stack). Can also be set via credential file profile or
              the C(AZURE_CLOUD_ENVIRONMENT) environment variable. If setting to a URL, you must also set the ARM_CLOUD_METADATA_URL if the cloud
              is not registered with azcli.
        type: str
        default: AzureCloud
        version_added: '0.0.1'
        env:
          - name: AZURE_CLOUD_ENVIRONMENT
    auth_source:
        description:
            - Controls the source of the credentials to use for authentication.
            - Can also be set via the C(ANSIBLE_AZURE_AUTH_SOURCE) environment variable.
            - When set to C(auto) (the default) the precedence is module parameters -> C(env) -> C(credential_file) -> C(cli).
            - When set to C(env), the credentials will be read from the environment variables
            - When set to C(cli), the credentials will be sources from the Azure CLI profile. C(subscription_id) or the environment variable
              C(AZURE_SUBSCRIPTION_ID) can be used to identify the subscription ID if more than one is present otherwise the default
              az cli subscription is used.
            - When set to C(msi), the host machine must be an azure resource with an enabled MSI extension. C(subscription_id) or the
              environment variable C(AZURE_SUBSCRIPTION_ID) can be used to identify the subscription ID if the resource is granted
              access to more than one subscription, otherwise the first subscription is chosen.
            - The C(msi) was added in Ansible 2.6.
        type: str
        default: auto
        choices:
        - auto
        - cli
        - env
        - msi
        env:
          - name: ANSIBLE_AZURE_AUTH_SOURCE
requirements:
    - python >= 2.7
    - The host that executes this module must have the azure.azcollection collection installed via galaxy
    - All python packages listed in collection's requirements.txt must be installed via pip on the host that executes modules from azure.azcollection
    - Full installation instructions may be found https://galaxy.ansible.com/azure/azcollection

notes:
    - For authentication with Azure you can pass parameters, set environment variables, use a profile stored
      in ~/.azure/credentials, or log in before you run your tasks or playbook with C(az login).
    - Authentication is also possible using a service principal.
    - To authenticate via OIDC, pass client_id, tenant and oidc_token or oidc_token_file_path, or set environment
      variables AZURE_CLIENT_ID, AZURE_TENANT and AZURE_FEDERATED_TOKEN or AZURE_FEDERATED_TOKEN_FILE.
    - To authenticate via service principal, pass subscription_id, client_id, secret and tenant or set environment
      variables AZURE_SUBSCRIPTION_ID, AZURE_CLIENT_ID, AZURE_SECRET and AZURE_TENANT.
    - "Alternatively, credentials can be stored in ~/.azure/credentials. This is an ini file containing
      a [default] section and the following keys: subscription_id, client_id, secret and tenant."

seealso:
    - name: Sign in with Azure CLI
      link: https://docs.microsoft.com/en-us/cli/azure/authenticate-azure-cli?view=azure-cli-latest
      description: How to authenticate using the C(az login) command.
    '''



# Copyright (c) 2015, Steve Gargan <steve.gargan@gmail.com>
# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later
from __future__ import annotations

DOCUMENTATION = r"""
author: Unknown (!UNKNOWN)
name: consul_kv
short_description: Fetch metadata from a Consul key value store
description:
  - Lookup metadata for a playbook from the key value store in a Consul cluster. Values can be easily set in the kv store
    with simple rest commands.
  - C(curl -X PUT -d 'some-value' http://localhost:8500/v1/kv/ansible/somedata).
requirements:
  - 'py-consul python library U(https://github.com/criteo/py-consul?tab=readme-ov-file#installation)'
options:
  _terms:
    description: List of key(s) to retrieve.
    type: list
    elements: string
  recurse:
    type: boolean
    description: If V(true), retrieves all the values that have the given key as prefix.
    default: false
  index:
    description:
      - If the key has a value with the specified index then this is returned allowing access to historical values.
    type: int
  datacenter:
    description:
      - Retrieve the key from a consul datacenter other than the default for the consul host.
    type: str
  token:
    description: The acl token to allow access to restricted values.
    type: str
  host:
    default: localhost
    type: str
    description:
      - The target to connect to, must be a resolvable address.
      - It is determined from E(ANSIBLE_CONSUL_URL) if that is set.
    ini:
      - section: lookup_consul
        key: host
  port:
    description:
      - The port of the target host to connect to.
      - If you use E(ANSIBLE_CONSUL_URL) this value is used from there.
    type: int
    default: 8500
  scheme:
    default: http
    type: str
    description:
      - Whether to use http or https.
      - If you use E(ANSIBLE_CONSUL_URL) this value is used from there.
  validate_certs:
    default: true
    description:
      - Whether to verify the TLS connection or not.
      - Instead of setting this to V(false), please consider using O(ca_path) instead.
    type: bool
    env:
      - name: ANSIBLE_CONSUL_VALIDATE_CERTS
    ini:
      - section: lookup_consul
        key: validate_certs
  ca_path:
    description: The CA bundle to use for HTTPS connections.
    type: str
    version_added: "12.6.0"
    env:
      - name: ANSIBLE_CONSUL_CA_PATH
    ini:
      - section: lookup_consul
        key: ca_path
  client_cert:
    description: The client cert to verify the TLS connection.
    type: str
    env:
      - name: ANSIBLE_CONSUL_CLIENT_CERT
    ini:
      - section: lookup_consul
        key: client_cert
  url:
    description:
      - The target to connect to.
      - 'Should look like this: V(https://my.consul.server:8500).'
    type: str
    version_added: 1.0.0
    env:
      - name: ANSIBLE_CONSUL_URL
    ini:
      - section: lookup_consul
        key: url
  empty_value:
    description:
      - Controls what is returned when a Consul value is null.
    type: str
    default: 'textual_none'
    choices:
      textual_none: Return the string V(None). This is the legacy behavior.
      python_none: Return a Python V(null)/V(None) value.
      empty_string: Return an empty string.
    version_added: 13.1.0
"""

EXAMPLES = r"""
- ansible.builtin.debug:
    msg: 'key contains {{item}}'
  with_community.general.consul_kv:
    - 'key/to/retrieve'

- name: Parameters can be provided after the key be more specific about what to retrieve
  ansible.builtin.debug:
    msg: 'key contains {{item}}'
  with_community.general.consul_kv:
    - 'key/to recurse=true token=E6C060A9-26FB-407A-B83E-12DDAFCB4D98'

- name: retrieving a KV from a remote cluster on non default port
  ansible.builtin.debug:
    msg: "{{ lookup('community.general.consul_kv', 'my/key', host='10.10.10.10', port=2000) }}"
"""

RETURN = r"""
_raw:
  description:
    - Value(s) stored in consul.
  type: dict
"""

from urllib.parse import urlparse

from ansible.errors import AnsibleAssertionError, AnsibleError
from ansible.module_utils.common.text.converters import to_text
from ansible.plugins.lookup import LookupBase

from ansible_collections.community.general.plugins.plugin_utils._lookup import check_for_wrong_terms

try:
    import consul

    HAS_CONSUL = True
except ImportError:
    HAS_CONSUL = False


_EMPTY_VALUE_MAP = {
    "textual_none": "None",
    "python_none": None,
    "empty_string": "",
}


class LookupModule(LookupBase):
    def run(self, terms, variables=None, **kwargs):
        if not HAS_CONSUL:
            raise AnsibleError(
                "py-consul is required for consul_kv lookup. see https://github.com/criteo/py-consul?tab=readme-ov-file#installation"
            )

        # get options
        self.set_options(direct=kwargs)
        check_for_wrong_terms(self, direct=kwargs)

        scheme = self.get_option("scheme")
        host = self.get_option("host")
        port = self.get_option("port")
        url = self.get_option("url")
        if url is not None:
            u = urlparse(url)
            if u.scheme:
                scheme = u.scheme
            host = u.hostname
            if u.port is not None:
                port = u.port

        validate_certs = self.get_option("validate_certs")
        ca_path = self.get_option("ca_path")
        client_cert = self.get_option("client_cert")

        verify = (ca_path or validate_certs) if validate_certs else False

        empty_value = _EMPTY_VALUE_MAP[self.get_option("empty_value")]
        values = []
        try:
            for term in terms:
                params = self.parse_params(term)
                consul_api = consul.Consul(host=host, port=port, scheme=scheme, verify=verify, cert=client_cert)

                results = consul_api.kv.get(
                    params["key"],
                    token=params["token"],
                    index=params["index"],
                    recurse=params["recurse"],
                    dc=params["datacenter"],
                )
                if results[1]:
                    # responds with a single or list of result maps
                    if isinstance(results[1], list):
                        for r in results[1]:
                            v = r["Value"]
                            values.append(to_text(v) if v is not None else empty_value)
                    else:
                        v = results[1]["Value"]
                        values.append(to_text(v) if v is not None else empty_value)
        except Exception as e:
            raise AnsibleError(f"Error locating '{term}' in kv store. Error was {e}") from e

        return values

    def parse_params(self, term):
        params = term.split(" ")

        paramvals = {
            "key": params[0],
            "token": self.get_option("token"),
            "recurse": self.get_option("recurse"),
            "index": self.get_option("index"),
            "datacenter": self.get_option("datacenter"),
        }

        # parameters specified?
        try:
            for param in params[1:]:
                if param and len(param) > 0:
                    name, value = param.split("=")
                    if name not in paramvals:
                        raise AnsibleAssertionError(f"{name} not a valid consul lookup parameter")
                    paramvals[name] = value
        except (ValueError, AssertionError) as e:
            raise AnsibleError(e) from e

        return paramvals



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_devices_packet_capture_captures_bulk_create
short_description: Resource module for organizations _devices _packet _capture _captures
  _bulk _create
description:
  - Manage operation create of the resource organizations _devices _packet _capture
    _captures _bulk _create.
  - Perform a packet capture on multiple devices and store in Meraki Cloud.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  advanced:
    description: Advanced capture options (optional).
    suboptions:
      bufferFiles:
        description: Number of buffer files for circular buffer capture (1-5 for Campus
          Gateway devices).
        type: int
      captureType:
        description: Type of capture. Possible values linear (default), circular.
        type: str
      controlPlaneDirection:
        description: Direction for control plane capture.
        type: str
      innerFilterMac:
        description: Inner MAC address filter for tunneled traffic (up to 5 MAC addresses
          for Campus Gateway devices).
        elements: str
        type: list
      maxFilesize:
        description: Maximum file size in megabytes (MB). Range 1-100 MB when bufferFiles=1,
          1-500 MB when bufferFiles=2-5.
        type: int
      physicalInterfaceDirection:
        description: Direction for physical interface capture.
        type: str
    type: dict
  devices:
    description: Device details (maximum of 20 devices allowed).
    elements: dict
    suboptions:
      interface:
        description: Interfaces to capture.
        type: str
      serial:
        description: Serial of the devices to schedule packet capture.
        type: str
      switchports:
        description: Switchports to capture.
        type: str
    type: list
  duration:
    description: Duration of the capture in seconds.
    type: int
  filterExpression:
    description: Filter expression for the capture.
    type: str
  name:
    description: Name of packet capture file.
    type: str
  notes:
    description: Reason for capture.
    type: str
  organizationId:
    description: OrganizationId path parameter. Organization ID.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for organizations bulkOrganizationDevicesPacketCaptureCapturesCreate
    description: Complete reference of the bulkOrganizationDevicesPacketCaptureCapturesCreate
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!bulk-organization-devices-packet-capture-captures-create
notes:
  - SDK Method used are
    organizations.Organizations.bulk_organization_devices_packet_capture_captures_create,
  - Paths used are
    post /organizations/{organizationId}/devices/packetCapture/captures/bulkCreate,
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.organizations_devices_packet_capture_captures_bulk_create:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    advanced:
      bufferFiles: 5
      captureType: circular
      controlPlaneDirection: both
      innerFilterMac:
        - aa:bb:cc:dd:ee:ff
        - 11:22:33:44:55:66
      maxFilesize: 10
      physicalInterfaceDirection: both
    devices:
      - interface: TenGigabitEthernet0/0/0
        serial: Q234-ABCD-5678
        switchports: 1, 2
    duration: 60
    filterExpression: (icmp)
    name: Capture no. 3
    notes: Debugging persistent issue on device
    organizationId: string
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "items": [
        {
          "captureId": "string",
          "network": {
            "id": "string",
            "name": "string"
          },
          "devices": [
            {}
          ],
          "device": {
            "name": "string",
            "serial": "string"
          },
          "admin": {
            "id": "string",
            "name": "string"
          },
          "client": {
            "id": "string",
            "mac": "string"
          },
          "details": [
            {
              "name": "string",
              "value": "string",
              "productType": "string"
            }
          ],
          "name": "string",
          "startTs": "string",
          "ports": "string",
          "status": "string",
          "errorMessage": "string",
          "destination": "string",
          "process": "string",
          "file": {
            "size": 0
          },
          "duration": 0,
          "filterExpression": "string",
          "counts": {
            "packets": {
              "total": 0
            }
          },
          "interface": "string"
        }
      ]
    }
"""
